`timescale 1ns / 1ps
// synthesis attribute rom_style of cnfg_mod is distributed

module cnfg_mod(CLK, RST, PU_RSTR_MACS, PU_RSTR_DFLT, PU_RSTR_CNFG, HRDW_INI, RCV_BG, XMIT_BG,
                CNFG_FUNCT, CNFG_VALDAT, CNFG_EXEC,
                FL_CNFG_NOW_DONE, FL_CNFG_ODV, RCV_OE, TX_OE, 
                CNFG_BR, FUNCT_DONE, PH2_DONE, HRDW_INI_DONE, VALID_CNUM,
                ETHER_CNFG, EXT_FIFO_CNFG, RESET_CNFG, VME_BGTO, VME_BTO, VME_CNFG, 
                WADDR_TYPE, MWE, CNFG_RCV_ODV, CNFG_TX_VALDAT, CNFG_SND_PKT, CNFG_TXTYPE,
                MACBUS, RCV_INOUT, TX_INOUT);

    input CLK;
    input RST;
    input PU_RSTR_MACS;
    input PU_RSTR_DFLT;
    input PU_RSTR_CNFG;
    input HRDW_INI;
    input RCV_BG;
    input XMIT_BG;
    input [4:0] CNFG_FUNCT;
    input CNFG_VALDAT;
    input CNFG_EXEC;
    input FL_CNFG_NOW_DONE;
    input FL_CNFG_ODV;
    input RCV_OE;
    input TX_OE;
   output [1:0] CNFG_BR;
   output FUNCT_DONE;
   output PH2_DONE;
   output HRDW_INI_DONE;
   output VALID_CNUM;
   output [15:0] ETHER_CNFG;
   output [15:0] EXT_FIFO_CNFG;
   output [15:0] RESET_CNFG;
   output [15:0] VME_BGTO;
   output [15:0] VME_BTO;
   output [31:0] VME_CNFG;
   output [4:0] WADDR_TYPE;
   output MWE;
   output CNFG_RCV_ODV;
   output CNFG_TX_VALDAT;
   output CNFG_SND_PKT;
   output [7:0] CNFG_TXTYPE;
    inout [15:0] MACBUS;
    inout [15:0] RCV_INOUT;
    inout [15:0] TX_INOUT;

reg [1:0] CNFG_BR;
reg FUNCT_DONE;
reg PH2_DONE;
reg [15:0] ETHER_CNFG;
reg [15:0] EXT_FIFO_CNFG;
reg [15:0] RESET_CNFG;
reg [15:0] VME_BGTO;
reg [15:0] VME_BTO;
reg [31:0] VME_CNFG;
reg [4:0] WADDR_TYPE;
reg MWE;
reg CNFG_RCV_ODV;
reg CNFG_TX_VALDAT;
reg CNFG_SND_PKT;
wire [7:0] CNFG_TXTYPE;
wire [15:0] MACBUS;
wire [15:0] RCV_INOUT;
wire [15:0] TX_INOUT;
reg [15:0] MACBUS_IN_REG;
reg [15:0] MACBUS_OUT_REG;
reg [15:0] RCV_IN_REG;
reg [15:0] RCV_OUT_REG;
reg [15:0] TX_IN_REG;
reg [15:0] TX_OUT_REG;
reg [4:0] CFR;
reg FL_CND;
reg FL_CDV;
reg LD_CFR;

reg CVD;
reg CMPLTD;
reg PUSH_RAW;
reg READ_RAW;
wire RAW_MT;   
wire RAW_FULL;   
wire RAW_AMT;   
wire RAW_AF;   
wire RAW_WTERR;   
wire RAW_RDERR;
wire [11:0] RWD1,RWD2,RWP1,RWP2;
wire [3:0] RAW_CNT;
wire [11:0] DCD_PAR1,DCD_PAR2;
wire [23:0] DCD_DATA;

reg [1:0] F2C_ST;
reg [6:0] AD_CNT;
reg LD_AC;
reg SHIFT;
reg [23:0] PARITY;
reg [23:0] DATA;
wire FILL;
wire BT0;
reg  BT0_DLY;
reg  [2:0] SIXB_PHASE;
wire VP1,VP2,BTX1,BTX2,CRPTD;    //VP = Valid Parity, BTX = Bad Transmision, CRPTD = Corrupted.
wire VLD_OUT;

reg  ST_CLR;
reg  [1:0] CR_ID;
reg  [31:0] BMASK;
reg  [2:0] SEL_CR;
reg  [5:0] CNFG_ST;
reg  [4:0] CNFG_NUM;
reg  [1:0] ACNT;
reg  [3:0] MAC_ID;
reg  [47:0] MAC_REG;
reg  [11:0] ECC_MUXH, ECC_MUXL;
reg  [11:0] ECC_MUXH1, ECC_MUXL1;
reg  [47:0] FL_DATA;
wire [11:0] ECC_PARH, ECC_PARL;
wire [5:0] BA;
reg  [5:0] M_OFFSET;
reg  [5:0] RELADDR;
reg  [15:0] HDR_REG;
reg  [6:0] ST_PG;
reg  [2:0] RND;
wire RND_RST;
wire RA_RST;
wire ACNT_RST;
wire SEL_CR_RST;
wire DCD_CNTR_RST;
wire LP_RST;
reg  RST_RND;
reg  RST_RA;
reg  RST_MAC_ID;
reg  RST_ACNT;
reg  RST_SEQ;
reg  RST_DCD_CNTR;
reg  RST_LP;
reg  INC_RND;
reg  RA_ENA;
reg  ACNT_ENA;
reg  MAC1,MAC2;
reg  MAC_WE;
reg  MAC_WE2;
reg  LD1_H,LD1_M,LD1_L;
reg  LD2_H,LD2_M,LD2_L;
reg  LD3_H,LD3_M,LD3_L;
reg  LD_H,LD_M,LD_L;
reg  LD_HDR1,LD_HDR2A,LD_HDR2B,LD_HDR2C;
reg  LOAD_MASK1,LOAD_MASK2;
reg  LD_MAC_ID;
reg  LD_CNFG_NUM;
reg  LD_CNFG_NUM_DFLT;
reg  ENA_RD_SEQ;
reg  ENA_WRT_SEQ;
reg  SET_CLR_BITS;
reg  LD_CR1,LD_CR2;
wire LONG_REG;
wire LD_FIRST;
wire LD_LAST;
wire LD_DCD_DONE;
reg  DCD_CNTR_ENA;
reg  [3:0] DCD_CNTR;
reg  LD_MAC_DONE;
reg  S1,S2,S3;
reg  LP;
reg  INCR_LP;
reg  LD_CR_ID;
reg  SET_MAC_ID;
reg  MID_ENA,MID_INC;
reg  ET_FL_LD;
reg  EF_FL_LD;
reg  RST_FL_LD;
reg  VME_FL_LD;
reg  BTO_FL_LD;
reg  BGTO_FL_LD;
reg  [47:0] CNFG_DCD_REG;
reg  LD_DCD_A,LD_DCD_B;
reg  RD_RW1; 
reg  RQ_RCV;
reg  RQ_TX;
reg  RL_BUS;

reg  HEAD_OUT;
reg  BA_DATA_OUT;

reg  [3:0] SM1_ST;
reg  SND_MAC_ENA;
reg  HEAD_OUT_MAC;
reg  BA_DATA_OUT_MAC;
reg  SM1_SD1;
reg  LD_FL_DATA_MAC;
reg  SH_FL_DATA_MAC;
reg  DV_MAC;
reg  SND_MAC_DONE;
reg  HOLD1;
reg  HOLD2;
reg  HOLD3;
reg  SHIFT_DONE;

reg  [2:0] SD1_ST;
reg  SND_DFLT_ENA;
reg  DFLT;
reg  HEAD_OUT_DFLT;
reg  BA_DATA_OUT_DFLT;
reg  LD_FL_DATA_DFLT;
reg  SH_FL_DATA_DFLT;
reg  RA_ENA_DFLT;
reg  DV_DFLT;
reg  SND_DFLT_DONE;

reg  [3:0] SC1_ST;
reg  SND_CNFG_ENA;
reg  SC1_LD_HDR1;
reg  HEAD_OUT_CNFG;
reg  BA_DATA_OUT_CNFG;
reg  SC1_SD1;
reg  LD_FL_DATA_CNFG;
reg  SH_FL_DATA_CNFG;
reg  ENA_CNFG;
reg  RA_ENA_CNFG;
reg  DV_CNFG;
reg  SND_CNFG_DONE;

reg  CSP,CSP1;
reg  SND_DFLT;
reg  SND_CNFGA;
reg  SND_CNFGB;
reg  SND_MAC;

reg HRDW_INI_1;
reg [20:0] MAC_INI;
reg  [3:0] MAC_INI_CNT;
wire HRDW_INI_DONE;
wire VALID_CNUM;

wire ALL_MAC;
wire LAST_CR;
wire LAST_MAC;
wire MASK_LD_RCV1;
wire MASK_LD_RCV2;
wire SET_TST_FF;
wire CLR_TST_FF;
wire SET_ECC_ENA;
wire CLR_ECC_ENA;
wire SET_INJ_ERR;
wire CLR_INJ_ERR;
wire SET_WARNSHD;
wire CLR_WARNSHD;
wire SET_STRTPKT;
wire CLR_STRTPKT;

wire ET_BSET;
wire ET_BCLR;
wire EF_BSET;
wire EF_BCLR;
wire RST_BSET;
wire RST_BCLR;
wire VME_BSET;
wire VME_BCLR;
wire SC_CMD;

wire ET_DIR_LD;
wire EF_DIR_LD;
wire RST_DIR_LD;
wire VME_DIR_LD_H;
wire VME_DIR_LD_L;
wire BTO_DIR_LD;
wire BGTO_DIR_LD;

wire FL_WRT;
wire FL2CNFG;
wire [15:0] HDR1;
wire [15:0] HDR2A;
wire [15:0] HDR2B;
wire [15:0] HDR2C;

`ifdef HEADER
`else
`include "header.v"
`endif

assign CNFG_TXTYPE = 8'h0A;

assign HDR1 = 16'h0001; 
assign HDR2A = ALL_MAC ? {FL_WRT,7'h00,FL2CNFG,7'h3C} : {FL_WRT,7'h00,FL2CNFG,7'h0C};
assign HDR2B = {FL_WRT,7'h00,FL2CNFG,7'h04};
assign HDR2C = {FL_WRT,ST_PG,FL2CNFG,7'h1E};
assign FL_WRT = (CFR==`Save_Cnfg_Num) || (CFR==`Set_Cnfg_Dflt) || (CFR==`Set_MACs);
assign FL2CNFG = (CFR==`Read_Cnfg_Num_Dcd) || (CFR==`Rstr_Cnfg_Num) || (CFR==`Read_Cnfg_Dflt) || (CFR==`Read_MACs_Dcd) || PU_RSTR_MACS || PU_RSTR_DFLT || PU_RSTR_CNFG;

assign MASK_LD_RCV1 = LOAD_MASK1 && (CFR==`Set_Clr_CRs);
assign MASK_LD_RCV2 = LOAD_MASK2 && (CFR==`Set_Clr_CRs);
assign SET_TST_FF  = LOAD_MASK1 && (CFR==`Set_FF_Test);
assign CLR_TST_FF  = LOAD_MASK1 && (CFR==`Set_FF_VME);
assign SET_ECC_ENA = LOAD_MASK1 && (CFR==`ECC_enable);
assign CLR_ECC_ENA = LOAD_MASK1 && (CFR==`ECC_disable);
assign SET_INJ_ERR = LOAD_MASK1 && (CFR==`Set_Inj_Err);
assign CLR_INJ_ERR = LOAD_MASK1 && (CFR==`Rst_Inj_Err);
assign SET_WARNSHD = LOAD_MASK1 && (CFR==`Warn_On_Shdwn);
assign CLR_WARNSHD = LOAD_MASK1 && (CFR==`No_Warn_On_Shdwn);
assign SET_STRTPKT = LOAD_MASK1 && (CFR==`Snd_Startup_Pkt);
assign CLR_STRTPKT = LOAD_MASK1 && (CFR==`No_Startup_Pkt);

assign SC_CMD     = (CFR==`Set_FF_Test) || (CFR==`Set_FF_VME) ||
                    (CFR==`ECC_enable) || (CFR==`ECC_disable) ||
                    (CFR==`Set_Inj_Err) || (CFR==`Rst_Inj_Err) ||
                    (CFR==`Warn_On_Shdwn) || (CFR==`No_Warn_On_Shdwn) ||
                    (CFR==`Snd_Startup_Pkt) || (CFR==`No_Startup_Pkt);

assign ET_BSET    = SET_CLR_BITS && (ST_CLR==1) && (CR_ID==`Eth_CR) && (CFR==`Set_Clr_CRs);
assign ET_BCLR    = SET_CLR_BITS && (ST_CLR==0) && (CR_ID==`Eth_CR) && (CFR==`Set_Clr_CRs);
assign EF_BSET    = SET_CLR_BITS && ((CFR==`Set_FF_Test) || (CFR==`ECC_enable) || (CFR==`Set_Inj_Err) || ((ST_CLR==1) && (CR_ID==`Ext_FF_CR) && (CFR==`Set_Clr_CRs)));
assign EF_BCLR    = SET_CLR_BITS && ((CFR==`Set_FF_VME) || (CFR==`ECC_disable) || (CFR==`Rst_Inj_Err) || ((ST_CLR==0) && (CR_ID==`Ext_FF_CR) && (CFR==`Set_Clr_CRs)));
assign RST_BSET    = SET_CLR_BITS && ((CFR==`Warn_On_Shdwn) || (CFR==`Snd_Startup_Pkt) || ((ST_CLR==1) && (CR_ID==`Rst_Ena_CR) && (CFR==`Set_Clr_CRs)));
assign RST_BCLR    = SET_CLR_BITS && ((CFR==`No_Warn_On_Shdwn) || (CFR==`No_Startup_Pkt) || ((ST_CLR==0) && (CR_ID==`Rst_Ena_CR) && (CFR==`Set_Clr_CRs)));
assign VME_BSET    = SET_CLR_BITS && (ST_CLR==1) && (CR_ID==`VME_CR) && (CFR==`Set_Clr_CRs);
assign VME_BCLR    = SET_CLR_BITS && (ST_CLR==0) && (CR_ID==`VME_CR) && (CFR==`Set_Clr_CRs);

assign ET_DIR_LD    = (LD_CR1 && (CFR==`Wrt_Eth_CR)) || (ENA_WRT_SEQ && (SEL_CR == `Sel_Eth_CR));
assign EF_DIR_LD    = (LD_CR1 && (CFR==`Wrt_Ext_CR)) || (ENA_WRT_SEQ && (SEL_CR == `Sel_Ext_CR));
assign RST_DIR_LD   = (LD_CR1 && (CFR==`Wrt_Rst_CR)) || (ENA_WRT_SEQ && (SEL_CR == `Sel_Rst_CR));
assign VME_DIR_LD_H = (LD_CR1 && (CFR==`Wrt_VME_CR)) || (ENA_WRT_SEQ && (SEL_CR == `Sel_VMEh_CR));
assign VME_DIR_LD_L = (LD_CR2 && (CFR==`Wrt_VME_CR)) || (ENA_WRT_SEQ && (SEL_CR == `Sel_VMEl_CR));
assign BTO_DIR_LD   = (LD_CR1 && (CFR==`Wrt_BTO_CR)) || (ENA_WRT_SEQ && (SEL_CR == `Sel_BTO_CR));
assign BGTO_DIR_LD  = (LD_CR1 && (CFR==`Wrt_BGTO_CR)) || (ENA_WRT_SEQ && (SEL_CR == `Sel_BGTO_CR));

assign LAST_CR  = (SEL_CR == 3'd6);
assign LAST_MAC = ((MAC_ID < 4'h8) && (ACNT == 2'd2)) || ({MAC_ID,ACNT} == 6'b110010);
assign BA = M_OFFSET + RELADDR;
assign ALL_MAC = MAC_ID[3];

assign FILL = FL_CDV && (TX_IN_REG[13:8] == 6'd63);  // Address is last byte in page
assign BT0 = (SIXB_PHASE == 3'd0);
assign CRPTD = (!VP1 || !VP2 || BTX1 || BTX2) && VLD_OUT;   

assign MACBUS = MWE ? MACBUS_OUT_REG : 16'hZZZZ;
assign RCV_INOUT = RCV_OE ? RCV_OUT_REG : 16'hZZZZ;
assign TX_INOUT = TX_OE ? TX_OUT_REG : 16'hZZZZ;
assign LD_FIRST = (DCD_CNTR == 4'd5);
assign LD_LAST = (DCD_CNTR == 4'd6);
assign LD_DCD_DONE = (DCD_CNTR == 4'd6);
assign VLD_OUT = (DCD_CNTR == 4'd5) || (DCD_CNTR == 4'd6);
assign LONG_REG = (CFR==`Wrt_VME_CR) || ((CR_ID==`VME_CR) && (CFR==`Set_Clr_CRs));

assign SEL_CR_RST = RST_SEQ || RST;
assign RND_RST = RST_RND || RST;
assign RA_RST = RST_RA || RST;
assign LP_RST = RST_LP || RST;
assign ACNT_RST = RST_ACNT || RST;
assign DCD_CNTR_RST = RST_DCD_CNTR || RST;

assign HRDW_INI_DONE = (MAC_INI_CNT == 4'hE);
assign VALID_CNUM = (CNFG_NUM <= 5'd20);

// Input registers //
always @(posedge CLK)
begin
   MACBUS_IN_REG <= MACBUS;
   RCV_IN_REG <= RCV_INOUT;
   TX_IN_REG <= TX_INOUT;
   CVD <= CNFG_VALDAT;
   FL_CND <= FL_CNFG_NOW_DONE;
   FL_CDV <= FL_CNFG_ODV;
end
// Output registers //
always @(posedge CLK)
begin
   HRDW_INI_1 <= HRDW_INI;
   MWE <= MAC_WE || MAC_WE2 || HRDW_INI_1;
   if(HRDW_INI_1)
      WADDR_TYPE <= MAC_INI[20:16];
   else
      WADDR_TYPE <= {MAC_ID[2:0],ACNT};
end
always @(posedge CLK)
begin
   if(RST)
      MACBUS_OUT_REG <= 16'h0000;
   else
      if(MAC_WE)
         MACBUS_OUT_REG <= RCV_IN_REG;
      else if(HRDW_INI_1)
         MACBUS_OUT_REG <= MAC_INI[15:0];
      else if(MAC_WE2)
         case(ACNT)
            3'd0:
               MACBUS_OUT_REG <= CNFG_DCD_REG[47:32];
            3'd1:
               MACBUS_OUT_REG <= CNFG_DCD_REG[31:16];
            3'd2:
               MACBUS_OUT_REG <= CNFG_DCD_REG[15:0];
            default:
               MACBUS_OUT_REG <= CNFG_DCD_REG[47:32];
         endcase
      else
         MACBUS_OUT_REG <= 16'h0000;
end




always @(posedge CLK)
begin
   CNFG_TX_VALDAT <= ENA_RD_SEQ || SND_DFLT || SND_CNFGA || SND_CNFGB || SND_MAC;
   CSP1 <= CSP;
   CNFG_SND_PKT <= CSP1;
end

// Configuration Function Register
always @(posedge CLK or posedge RST)
begin
   if(RST)
      CFR <= 5'h00;
   else
      if(LD_CFR)
         CFR <= CNFG_FUNCT;
      else
         CFR <= CFR;
end

// Register for storing Configuration Number
always @(posedge CLK or posedge RST)
begin
   if(RST)
      CNFG_NUM <= 5'hFF;
   else
      if(LD_CNFG_NUM)
         CNFG_NUM <= RCV_IN_REG[4:0];
      else if(LD_CNFG_NUM_DFLT)
         CNFG_NUM <= DCD_DATA[4:0];
      else
         CNFG_NUM <= CNFG_NUM;
end


always @(posedge CLK)
begin
   RD_RW1 <= READ_RAW;
   S2 <= S1;
   S3 <= S2;
end

always @(posedge CLK or posedge DCD_CNTR_RST)
begin
   if(DCD_CNTR_RST)
      DCD_CNTR <= 4'd0;
   else
      if(DCD_CNTR_ENA)
         DCD_CNTR <= DCD_CNTR + 1;
      else
         DCD_CNTR <= DCD_CNTR;
end

always @(posedge CLK or posedge LP_RST)
begin
   if(LP_RST)
      LP <= 0;
   else
      if(INCR_LP)
         LP <= LP + 1;
      else
         LP <= LP;
end

// Bus Request logic
always @(posedge CLK or posedge RST)
begin
   if(RST)
      CNFG_BR <= `Cnfg_NoOp;
   else
      if(RQ_RCV)
         CNFG_BR <= `Cnfg_Store;
      else if(RQ_TX)
         CNFG_BR <= `Cnfg_Xmit;
      else if(RL_BUS)
         CNFG_BR <= `Cnfg_NoOp;
      else
         CNFG_BR <= CNFG_BR;
end


////////////////////////////////////////////////////////////////////
//                                                                //
// State Machine for controlling Configuration Register Functions //
//                                                                //
////////////////////////////////////////////////////////////////////
always @(posedge CLK or posedge RST)
begin: Cnfg_FSM
   if(RST)
      CNFG_ST <= `Cnfg_Idle;
   else
      case(CNFG_ST)
         `Cnfg_Idle:
            if(PU_RSTR_DFLT || PU_RSTR_CNFG)
               CNFG_ST <= `Req_Rcv_Bus;
            else if(PU_RSTR_MACS)
               CNFG_ST <= `Set_MAC_ID;
            else if(SC_CMD)
               CNFG_ST <= `Mask_LD1;
            else
               case(CFR)
                  `Read_CRs:
                     CNFG_ST <= `Req_Tx_Bus_Cnfg;
                  `Wrt_Eth_CR,`Wrt_Ext_CR,`Wrt_Rst_CR,`Wrt_VME_CR,`Wrt_BTO_CR,`Wrt_BGTO_CR:
                     CNFG_ST <= `W4ValDat_Wrt;
                  `Wrt_All_CRs:
                     CNFG_ST <= `W4ValDat_All;
                  `Set_Clr_CRs:
                     CNFG_ST <= `W4ValDat_SC;
                  `Save_Cnfg_Num,`Read_Cnfg_Num_Dir,`Read_Cnfg_Num_Dcd,`Rstr_Cnfg_Num,`Set_Cnfg_Dflt:
                     CNFG_ST <= `Get_Cnfg_Num;
                  `Set_MACs:
                     CNFG_ST <= `Get_MAC_ID;
                  `Read_MACs_Dir,`Read_MACs_Dcd:
                     CNFG_ST <= `Set_MAC_ID;
                  `Read_Cnfg_Dflt:
                     CNFG_ST <= `W4Exec;
                  default:
                     CNFG_ST <= `Cnfg_Idle;
               endcase
         `W4ValDat_Wrt:
            if(CVD && !LONG_REG)
               CNFG_ST <= `Funct_Done;
            else if(CVD && LONG_REG)
               CNFG_ST <= `Wrt_VME_L;
            else
               CNFG_ST <= `W4ValDat_Wrt;
         `Wrt_VME_L:
            CNFG_ST <= `Funct_Done;
         `W4ValDat_All:
            if(LAST_CR)
               CNFG_ST <= `Funct_Done;
            else
               CNFG_ST <= `W4ValDat_All;
         `W4ValDat_SC:
            if(CVD)
               CNFG_ST <= `Mask_LD1;
            else
               CNFG_ST <= `W4ValDat_SC;
         `Mask_LD1:
            if(LONG_REG)
               CNFG_ST <= `Mask_LD2;
            else
               CNFG_ST <= `Set_Clr_Bits;
         `Mask_LD2:
            CNFG_ST <= `Set_Clr_Bits;
         `Set_Clr_Bits:
            CNFG_ST <= `Funct_Done;
         `Get_Cnfg_Num:
            if(CVD)
               CNFG_ST <= `W4Exec;
            else
               CNFG_ST <= `Get_Cnfg_Num;
         `Get_MAC_ID:
            if(CVD)
               CNFG_ST <= `Wrt_MAC_Seq;
            else
               CNFG_ST <= `Get_MAC_ID;
         `Wrt_MAC_Seq:
            if(LAST_MAC)
               CNFG_ST <= `W4Exec;
            else
               CNFG_ST <= `Wrt_MAC_Seq;
         `Set_MAC_ID:
            if((CFR == `Read_MACs_Dir) || (CFR == `Read_MACs_Dcd))
               CNFG_ST <= `W4Exec;
            else
               CNFG_ST <= `Req_Rcv_Bus;
         `Req_Rcv_Bus:
            if(RCV_BG)
               CNFG_ST <= `W4Exec;
            else
               CNFG_ST <= `Req_Rcv_Bus;
         `W4Exec:
            if(PU_RSTR_MACS)
               CNFG_ST <= `Snd_MAC_Seq;
            else if(PU_RSTR_DFLT)
               CNFG_ST <= `Snd_Dflt_Seq;
            else if(PU_RSTR_CNFG)
               CNFG_ST <= `Snd_Cnfg_Seq;
            else
               if(CNFG_EXEC)
                  case(CFR)
                     `Save_Cnfg_Num,`Read_Cnfg_Num_Dir,`Read_Cnfg_Num_Dcd,`Rstr_Cnfg_Num:
                        CNFG_ST <= `Snd_Cnfg_Seq;
                     `Set_MACs:
                        CNFG_ST <= `Get_MAC_Seq;
                     `Read_MACs_Dir,`Read_MACs_Dcd:
                        CNFG_ST <= `Snd_MAC_Seq;
                     `Set_Cnfg_Dflt,`Read_Cnfg_Dflt:
                        CNFG_ST <= `Snd_Dflt_Seq;
                     default:
                        CNFG_ST <= `W4Exec;
                  endcase
               else
                  CNFG_ST <= `W4Exec;
         `Get_MAC_Seq:
            if(ACNT == 2'd2)
               CNFG_ST <= `Ld_MAC_Reg;
            else
               CNFG_ST <= `Get_MAC_Seq;
         `Ld_MAC_Reg:
            if(LD_MAC_DONE)
               CNFG_ST <= `Snd_MAC_Seq;
            else
               CNFG_ST <= `Ld_MAC_Reg;
         `Snd_MAC_Seq:
            if(SND_MAC_DONE)
               if(ALL_MAC && (MAC_ID[2:0] != 3'h4))
                  CNFG_ST <= `Incr_MAC_ID;
               else if(!FL_WRT && FL2CNFG)
                  if(PU_RSTR_MACS)
                     CNFG_ST <= `Rel_Rx_Bus_MAC;
                  else
                     CNFG_ST <= `MAC_PH1Done;
               else
                  CNFG_ST <= `Funct_Done;
            else
               CNFG_ST <= `Snd_MAC_Seq;
         `Incr_MAC_ID:
            if(FL_WRT)
               CNFG_ST <= `Get_MAC_Seq;
            else
               CNFG_ST <= `Snd_MAC_Seq;
         `Snd_Dflt_Seq:
            if(SND_DFLT_DONE)
               if(!FL_WRT && FL2CNFG)
                  if(PU_RSTR_DFLT)
                     CNFG_ST <= `Rel_Rx_Bus_Dflt;
                  else
                     CNFG_ST <= `Dflt_PH1Done;
               else
                  CNFG_ST <= `Funct_Done;
            else
               CNFG_ST <= `Snd_Dflt_Seq;
         `Snd_Cnfg_Seq:
            if(SND_CNFG_DONE)
               if(!FL_WRT && FL2CNFG)
                  if(PU_RSTR_CNFG)
                     CNFG_ST <= `Rel_Rx_Bus_Cnfg;
                  else
                     CNFG_ST <= `Cnfg_PH1Done;
               else
                  CNFG_ST <= `Funct_Done;
            else
               CNFG_ST <= `Snd_Cnfg_Seq;
         `Rel_Rx_Bus_MAC:
            if(!RCV_BG)
               CNFG_ST <= `Rcv_MAC_FL;
            else
               CNFG_ST <= `Rel_Rx_Bus_MAC;
         `MAC_PH1Done:
            if(CNFG_FUNCT == `Funct_NoOp)
               CNFG_ST <= `Rcv_MAC_FL;
            else
               CNFG_ST <= `MAC_PH1Done;
         `Rel_Rx_Bus_Dflt:
            if(!RCV_BG)
               CNFG_ST <= `Rcv_Dflt_FL;
            else
               CNFG_ST <= `Rel_Rx_Bus_Dflt;
         `Dflt_PH1Done:
            if(CNFG_FUNCT == `Funct_NoOp)
               CNFG_ST <= `Rcv_Dflt_FL;
            else
               CNFG_ST <= `Dflt_PH1Done;
         `Rel_Rx_Bus_Cnfg:
            if(!RCV_BG)
               CNFG_ST <= `Rcv_Cnfg_FL;
            else
               CNFG_ST <= `Rel_Rx_Bus_Cnfg;
         `Cnfg_PH1Done:
            if(CNFG_FUNCT == `Funct_NoOp)
               CNFG_ST <= `Rcv_Cnfg_FL;
            else
               CNFG_ST <= `Cnfg_PH1Done;
         `Rcv_MAC_FL:
            if(CMPLTD)
               if(PU_RSTR_MACS)
                  CNFG_ST <= `Read_2_MAC;
               else
                  CNFG_ST <= `Req_Tx_Bus_MAC;
            else
               CNFG_ST <= `Rcv_MAC_FL;
         `Rcv_Dflt_FL:
            if(CMPLTD)
               if(PU_RSTR_DFLT)
                  CNFG_ST <= `Read_1_Dflt;
               else
                  CNFG_ST <= `Req_Tx_Bus_Dflt;
            else
               CNFG_ST <= `Rcv_Dflt_FL;
         `Rcv_Cnfg_FL:
            if(CMPLTD)
               if(PU_RSTR_CNFG || (CFR == `Rstr_Cnfg_Num))
                  CNFG_ST <= `Read_2_Cnfg;
               else
                  CNFG_ST <= `Req_Tx_Bus_Cnfg;
            else
               CNFG_ST <= `Rcv_Cnfg_FL;
         `Req_Tx_Bus_MAC:
            if(XMIT_BG)
               CNFG_ST <= `Read_2_MAC;
            else
               CNFG_ST <= `Req_Tx_Bus_MAC;
         `Read_2_MAC:
            if(RD_RW1)
               CNFG_ST <= `Load_2_MAC;
            else
               CNFG_ST <= `Read_2_MAC;
         `Req_Tx_Bus_Dflt:
            if(XMIT_BG)
               CNFG_ST <= `Read_1_Dflt;
            else
               CNFG_ST <= `Req_Tx_Bus_Dflt;
         `Read_1_Dflt:
            CNFG_ST <= `Load_Dflt;
         `Req_Tx_Bus_Cnfg:
            if(XMIT_BG && (CFR == `Read_CRs))
               CNFG_ST <= `Ena_Rd_Seq;
            else if(XMIT_BG)
               CNFG_ST <= `Read_2_Cnfg;
            else
               CNFG_ST <= `Req_Tx_Bus_Cnfg;
         `Ena_Rd_Seq:
            if(LAST_CR)
               CNFG_ST <= `Cnfg_Snd_Pkt;
            else
               CNFG_ST <= `Ena_Rd_Seq;
         `Read_2_Cnfg:
            if(RD_RW1)
               CNFG_ST <= `Load_2_Cnfg;
            else
               CNFG_ST <= `Read_2_Cnfg;
         `Load_2_MAC:
            if(LD_DCD_DONE)
               if(PU_RSTR_MACS)
                  CNFG_ST <= `Wrt_MAC_Seq2;
               else
                  CNFG_ST <= `Send_MAC;
            else
               CNFG_ST <= `Load_2_MAC;
         `Load_Dflt:
            if(LD_DCD_DONE)
               if(PU_RSTR_DFLT)
                  CNFG_ST <= `PH2_Done;
               else
                  CNFG_ST <= `Send_Dflt;
            else
               CNFG_ST <= `Load_Dflt;
         `Load_2_Cnfg:
            if(LD_DCD_DONE)
               if(PU_RSTR_CNFG || (CFR == `Rstr_Cnfg_Num))
                  CNFG_ST <= `Load_CRsA;
               else
                  CNFG_ST <= `Send_CnfgA;
            else
               CNFG_ST <= `Load_2_Cnfg;
         `Wrt_MAC_Seq2:
            if(LAST_MAC)
               CNFG_ST <= `PH2_Done;
            else
               if(ACNT == 2'd2)
                  CNFG_ST <= `Incr_MAC_ID2;
               else
                  CNFG_ST <= `Wrt_MAC_Seq2;
         `Send_MAC:
            if(LAST_MAC)
               CNFG_ST <= `Cnfg_Snd_Pkt;
            else
               if(ACNT == 2'd2)
                  CNFG_ST <= `Incr_MAC_ID2;
               else
                  CNFG_ST <= `Send_MAC;
         `Send_Dflt:
            CNFG_ST <= `Cnfg_Snd_Pkt;
         `Load_CRsA:
            if(LP == 1)
               CNFG_ST <= `Read_1_Cnfg;
            else
               CNFG_ST <= `Incr_Lp;
         `Send_CnfgA:
            if(S3)
               if(LP == 1)
                  CNFG_ST <= `Read_1_Cnfg;
               else
                  CNFG_ST <= `Incr_Lp;
            else
               CNFG_ST <= `Send_CnfgA;
         `Incr_MAC_ID2:
            CNFG_ST <= `Read_2_MAC;
         `Incr_Lp:
            CNFG_ST <= `Read_2_Cnfg;
         `Read_1_Cnfg:
            CNFG_ST <= `Load_1_Cnfg;
         `Load_1_Cnfg:
            if(LD_DCD_DONE)
               if(PU_RSTR_CNFG || (CFR == `Rstr_Cnfg_Num))
                  CNFG_ST <= `Load_CRsB;
               else
                  CNFG_ST <= `Send_CnfgB;
            else
               CNFG_ST <= `Load_1_Cnfg;
         `Load_CRsB:
            CNFG_ST <= `PH2_Done;
         `Send_CnfgB:
            CNFG_ST <= `Cnfg_Snd_Pkt;
         `Cnfg_Snd_Pkt:
            CNFG_ST <= `Rel_Tx_Bus;
         `Rel_Tx_Bus:
            if(!XMIT_BG)
               CNFG_ST <= `PH2_Done;
            else
               CNFG_ST <= `Rel_Tx_Bus;
         `PH2_Done:
            CNFG_ST <= `Cnfg_Idle;
         `Funct_Done:
            if(CFR == `Funct_NoOp)
               CNFG_ST <= `Cnfg_Idle;
            else
               CNFG_ST <= `Funct_Done;
         default:
            CNFG_ST <= `Cnfg_Idle;
      endcase
end

always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Cnfg_Idle,`Funct_Done,`PH2_Done:
         LD_CFR <= 1;
      default:
         LD_CFR <= 0;
   endcase
end
always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Get_Cnfg_Num:
         LD_CNFG_NUM <= 1;
      default:
         LD_CNFG_NUM <= 0;
   endcase
end

// For standalone simulation only
//always @(CNFG_ST)
//begin
//   case(CNFG_ST)
//      `W4_Rd_CRs:
//         begin
//            W4RDCRS <= 1;
//            W4EXEC <= 0;
//         end
//      `W4Exec:
//         begin
//            W4RDCRS <= 0;
//            W4EXEC <= 1;
//         end
//      default:
//         begin
//            W4RDCRS <= 0;
//            W4EXEC <= 0;
//         end
//   endcase
//end

always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Set_MAC_ID:
         begin
            SET_MAC_ID <= 1;
            LD_MAC_ID <= 0;
            MID_INC <= 0;
            MID_ENA <= 0;
            ACNT_ENA <= 0;
            MAC_WE <= 0;
            MAC_WE2 <= 0;
         end
      `Get_MAC_ID:
         begin
            SET_MAC_ID <= 0;
            LD_MAC_ID <= 1;
            MID_INC <= 0;
            MID_ENA <= 0;
            ACNT_ENA <= 0;
            MAC_WE <= 0;
            MAC_WE2 <= 0;
         end
      `Wrt_MAC_Seq:
         begin
            SET_MAC_ID <= 0;
            LD_MAC_ID <= 0;
            MID_INC <= 0;
            MID_ENA <= 1;
            ACNT_ENA <= 1;
            MAC_WE <= 1;
            MAC_WE2 <= 0;
         end
      `Get_MAC_Seq:
         begin
            SET_MAC_ID <= 0;
            LD_MAC_ID <= 0;
            MID_INC <= 0;
            MID_ENA <= 0;
            ACNT_ENA <= 1;
            MAC_WE <= 0;
            MAC_WE2 <= 0;
         end
      `Wrt_MAC_Seq2:
         begin
            SET_MAC_ID <= 0;
            LD_MAC_ID <= 0;
            MID_INC <= 0;
            MID_ENA <= 0;
            ACNT_ENA <= 1;
            MAC_WE <= 0;
            MAC_WE2 <= 1;
         end
      `Send_MAC:
         begin
            SET_MAC_ID <= 0;
            LD_MAC_ID <= 0;
            MID_INC <= 0;
            MID_ENA <= 0;
            ACNT_ENA <= 1;
            MAC_WE <= 0;
            MAC_WE2 <= 0;
         end
      `Incr_MAC_ID,`Incr_MAC_ID2:
         begin
            SET_MAC_ID <= 0;
            LD_MAC_ID <= 0;
            MID_INC <= 1;
            MID_ENA <= 0;
            ACNT_ENA <= 0;
            MAC_WE <= 0;
            MAC_WE2 <= 0;
         end
      default:
         begin
            SET_MAC_ID <= 0;
            LD_MAC_ID <= 0;
            MID_INC <= 0;
            MID_ENA <= 0;
            ACNT_ENA <= 0;
            MAC_WE <= 0;
            MAC_WE2 <= 0;
         end
   endcase
end


always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Cnfg_Snd_Pkt:
         CSP <= 1;
      default:
         CSP <= 0;
   endcase
end

always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Send_Dflt:
         begin
            S1 <= 0;
            SND_DFLT <= 1;
            SND_CNFGA <= 0;
            SND_CNFGB <= 0;
            SND_MAC <= 0;
         end
      `Send_CnfgA:
         begin
            S1 <= 1;
            SND_DFLT <= 0;
            SND_CNFGA <= 1;
            SND_CNFGB <= 0;
            SND_MAC <= 0;
         end
      `Send_CnfgB:
         begin
            S1 <= 0;
            SND_DFLT <= 0;
            SND_CNFGA <= 0;
            SND_CNFGB <= 1;
            SND_MAC <= 0;
         end
      `Send_MAC:
         begin
            S1 <= 0;
            SND_DFLT <= 0;
            SND_CNFGA <= 0;
            SND_CNFGB <= 0;
            SND_MAC <= 1;
         end
      default:
         begin
            S1 <= 0;
            SND_DFLT <= 0;
            SND_CNFGA <= 0;
            SND_CNFGB <= 0;
            SND_MAC <= 0;
         end
   endcase
end

always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Read_1_Dflt,`Read_1_Cnfg,`Read_2_MAC,`Read_2_Cnfg:
         READ_RAW <= 1;
      default:
         READ_RAW <= 0;
   endcase
end
always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Cnfg_Idle,`W4Exec:
         begin
            RST_SEQ <= 1;
            RST_ACNT <= 1;
            RST_LP <= 1;
            RST_RA <= 1;
            RST_MAC_ID <= 1;
            RST_DCD_CNTR <= 1;
         end
      `Read_1_Dflt,`Read_1_Cnfg,`Read_2_MAC,`Read_2_Cnfg:
         begin
            RST_SEQ <= 0;
            RST_ACNT <= 0;
            RST_LP <= 0;
            RST_RA <= 0;
            RST_MAC_ID <= 0;
            RST_DCD_CNTR <= 1;
         end
      `Rcv_MAC_FL:
         begin
            RST_SEQ <= 0;
            RST_ACNT <= 1;
            RST_LP <= 0;
            RST_RA <= 0;
            RST_MAC_ID <= 1;
            RST_DCD_CNTR <= 0;
         end
      default:
         begin
            RST_SEQ <= 0;
            RST_ACNT <= 0;
            RST_LP <= 0;
            RST_RA <= 0;
            RST_MAC_ID <= 0;
            RST_DCD_CNTR <= 0;
         end
   endcase
end
always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Incr_Lp:
         begin
            INCR_LP <= 1;
         end
      default:
         begin
            INCR_LP <= 0;
         end
   endcase
end

always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Snd_MAC_Seq:
         begin
            SND_MAC_ENA <= 1;
            SND_CNFG_ENA <= 0;
            SND_DFLT_ENA <= 0;
         end
      `Snd_Cnfg_Seq:
         begin
            SND_MAC_ENA <= 0;
            SND_CNFG_ENA <= 1;
            SND_DFLT_ENA <= 0;
         end
      `Snd_Dflt_Seq:
         begin
            SND_MAC_ENA <= 0;
            SND_CNFG_ENA <= 0;
            SND_DFLT_ENA <= 1;
         end
      default:
         begin
            SND_MAC_ENA <= 0;
            SND_CNFG_ENA <= 0;
            SND_DFLT_ENA <= 0;
         end
   endcase
end

always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Req_Rcv_Bus:
         begin
            RQ_RCV <= 1;
            RQ_TX  <= 0;
            RL_BUS <= 0;
         end
      `Req_Tx_Bus_Dflt,`Req_Tx_Bus_Cnfg,`Req_Tx_Bus_MAC:
         begin
            RQ_RCV <= 0;
            RQ_TX  <= 1;
            RL_BUS <= 0;
         end
      `Rel_Tx_Bus,`Rel_Rx_Bus_Dflt,`Rel_Rx_Bus_Cnfg,`Rel_Rx_Bus_MAC:
         begin
            RQ_RCV <= 0;
            RQ_TX  <= 0;
            RL_BUS <= 1;
         end
      default:
         begin
            RQ_RCV <= 0;
            RQ_TX  <= 0;
            RL_BUS <= 0;
         end
   endcase
end

always @(CNFG_ST or LP)
begin
   case(CNFG_ST)
      `Load_CRsA:
         begin
            ET_FL_LD <= (LP == 1'b0);
            EF_FL_LD <= (LP == 1'b0);
            RST_FL_LD <= (LP == 1'b0);
            VME_FL_LD <= (LP == 1'b1);
            BTO_FL_LD <= (LP == 1'b1);
            BGTO_FL_LD <= 0;
         end
      `Load_CRsB:
         begin
            ET_FL_LD <= 0;
            EF_FL_LD <= 0;
            RST_FL_LD <= 0;
            VME_FL_LD <= 0;
            BTO_FL_LD <= 0;
            BGTO_FL_LD <= 1;
         end
      default:
         begin
            ET_FL_LD <= 0;
            EF_FL_LD <= 0;
            RST_FL_LD <= 0;
            VME_FL_LD <= 0;
            BTO_FL_LD <= 0;
            BGTO_FL_LD <= 0;
         end
   endcase
end

always @(CNFG_ST or CVD)
begin
   case(CNFG_ST)
      `Ena_Rd_Seq:
         begin
            ENA_RD_SEQ <= 1;
            ENA_WRT_SEQ <= 0;
         end
      `W4ValDat_All:
         begin
            ENA_RD_SEQ <= 0;
            ENA_WRT_SEQ <= CVD;
         end
      default:
         begin
            ENA_RD_SEQ <= 0;
            ENA_WRT_SEQ <= 0;
         end
   endcase
end

always @(CNFG_ST)
begin
   case(CNFG_ST)
      `Funct_Done,`Dflt_PH1Done,`Cnfg_PH1Done,`MAC_PH1Done:
         begin
            FUNCT_DONE <= 1;
            PH2_DONE <= 0;
         end
      `PH2_Done:
         begin
            FUNCT_DONE <= 0;
            PH2_DONE <= 1;
         end
      default:
         begin
            FUNCT_DONE <= 0;
            PH2_DONE <= 0;
         end
   endcase
end

always @(CNFG_ST or LD_FIRST or LD_LAST or PU_RSTR_DFLT)
begin
   case(CNFG_ST)
      `Load_2_Cnfg,`Load_2_MAC:
         begin
            LD_DCD_A <= LD_FIRST;
            LD_DCD_B <= LD_LAST;
            LD_CNFG_NUM_DFLT <= 0;
            DCD_CNTR_ENA <= 1;
         end
      `Load_1_Cnfg,`Load_Dflt:
         begin
            LD_DCD_A <= LD_LAST;
            LD_DCD_B <= 0;
            LD_CNFG_NUM_DFLT <= LD_LAST && PU_RSTR_DFLT;
            DCD_CNTR_ENA <= 1;
         end
      default:
         begin
            LD_DCD_A <= 0;
            LD_DCD_B <= 0;
            LD_CNFG_NUM_DFLT <= 0;
            DCD_CNTR_ENA <= 0;
         end
   endcase
end

always @(CNFG_ST or CVD)
begin
   case(CNFG_ST)
      `W4ValDat_Wrt:
         begin
            LD_CR1 <= CVD;
            LD_CR2 <= 0;
         end
      `Wrt_VME_L:
         begin
            LD_CR1 <= 0;
            LD_CR2 <= CVD;
         end
      default:
         begin
            LD_CR1 <= 0;
            LD_CR2 <= 0;
         end
   endcase
end

always @(CNFG_ST or CVD)
begin
   case(CNFG_ST)
      `W4ValDat_SC:
         begin
            LD_CR_ID <= CVD;
            LOAD_MASK1 <= 0;
            LOAD_MASK2 <= 0;
            SET_CLR_BITS <= 0;
         end
      `Mask_LD1:
         begin
            LD_CR_ID <= 0;
            LOAD_MASK1 <= 1;
            LOAD_MASK2 <= 0;
            SET_CLR_BITS <= 0;
         end
      `Mask_LD2:
         begin
            LD_CR_ID <= 0;
            LOAD_MASK1 <= 0;
            LOAD_MASK2 <= 1;
            SET_CLR_BITS <= 0;
         end
      `Set_Clr_Bits:
         begin
            LD_CR_ID <= 0;
            LOAD_MASK1 <= 0;
            LOAD_MASK2 <= 0;
            SET_CLR_BITS <= 1;
         end
      default:
         begin
            LD_CR_ID <= 0;
            LOAD_MASK1 <= 0;
            LOAD_MASK2 <= 0;
            SET_CLR_BITS <= 0;
         end
   endcase
end


//////////////////////////////////////////////////////////////////
//                                                              //
// Configuration registers for Ethernet, External FIFO,         //
// Reset Enables, and VME.                                      //
// (Functions are Reset, Load, Hold, BitSet, and BitClr)        //
// Also Bus Time Out and Bus Grant Time Out                     //
// (Functions are Reset, Load, and Hold)                        //
//                                                              //
//////////////////////////////////////////////////////////////////

always @(posedge CLK or posedge RST)
begin
   if(RST)
      BMASK <= 32'h00000000;
   else
      if(MASK_LD_RCV1)
         if(CR_ID==`VME_CR)
            BMASK <= {RCV_IN_REG,BMASK[15:0]};
         else
            BMASK <= {BMASK[31:16],RCV_IN_REG};
      else if(MASK_LD_RCV2)
         if(CR_ID==`VME_CR)
            BMASK <= {BMASK[31:16],RCV_IN_REG};
         else
            BMASK <= BMASK;
      else if(SET_TST_FF)
         BMASK <= `Test_FF_Mode_msk;
      else if(CLR_TST_FF)
         BMASK <= `VME_FF_Mode_msk;
      else if(SET_ECC_ENA)
         BMASK <= `ECC_Enable_msk;
      else if(CLR_ECC_ENA)
         BMASK <= `ECC_Disable_msk;
      else if(SET_INJ_ERR)
         BMASK <= `Set_Inj_Err_msk;
      else if(CLR_INJ_ERR)
         BMASK <= `Rst_Inj_Err_msk;
      else if(SET_WARNSHD)
         BMASK <= `Warn_On_Shdwn_msk;
      else if(CLR_WARNSHD)
         BMASK <= `No_Warn_On_Shdwn_msk;
      else if(SET_STRTPKT)
         BMASK <= `Snd_Startup_Pkt_msk;
      else if(CLR_STRTPKT)
         BMASK <= `No_Startup_Pkt_msk;
      else
         BMASK <= BMASK;
end

// Configure Register ID and Set/Clr register
always @(posedge CLK or posedge RST)
begin
   if(RST)
      begin
         CR_ID <= 2'b00;
         ST_CLR <= 0;
      end
   else
      if(LD_CR_ID)
         begin
            CR_ID <= RCV_IN_REG[1:0];
            ST_CLR <= RCV_IN_REG[8];
         end
      else
         begin
            CR_ID <= CR_ID;
            ST_CLR <= ST_CLR;
         end
end

// Register to hold Decoded register data
always @(posedge CLK or posedge RST)
begin
   if(RST)
      CNFG_DCD_REG <= 48'h000000000000;
   else
      if(LD_DCD_A)
         CNFG_DCD_REG <= {DCD_DATA,CNFG_DCD_REG[23:0]};
      else if(LD_DCD_B)
         CNFG_DCD_REG <= {CNFG_DCD_REG[47:24],DCD_DATA};
      else
         CNFG_DCD_REG <= CNFG_DCD_REG;
end


always @(posedge CLK or posedge RST)
begin
   if(RST)
      ETHER_CNFG <= `Ether_Cnfg_Dflt;  // default value
   else
      if(ET_DIR_LD)
         ETHER_CNFG <= RCV_IN_REG;
      else if(ET_FL_LD)
         ETHER_CNFG <= CNFG_DCD_REG[47:32];
      else if(ET_BSET)
         ETHER_CNFG <= BMASK[15:0] | ETHER_CNFG;
      else if(ET_BCLR)
         ETHER_CNFG <= BMASK[15:0] & ETHER_CNFG;
      else
         ETHER_CNFG <= ETHER_CNFG;
end

always @(posedge CLK or posedge RST)
begin
   if(RST)
      EXT_FIFO_CNFG <= `Ext_FIFO_Cnfg_Dflt;  // default value
   else
      if(EF_DIR_LD)
         EXT_FIFO_CNFG <= RCV_IN_REG;
      else if(EF_FL_LD)
         EXT_FIFO_CNFG <= CNFG_DCD_REG[31:16];
      else if(EF_BSET)
         EXT_FIFO_CNFG <= BMASK[15:0] | EXT_FIFO_CNFG;
      else if(EF_BCLR)
         EXT_FIFO_CNFG <= BMASK[15:0] & EXT_FIFO_CNFG;
      else
         EXT_FIFO_CNFG <= EXT_FIFO_CNFG;
end


always @(posedge CLK or posedge RST)
begin
   if(RST)
      RESET_CNFG <= `Reset_Cnfg_Dflt;  // default value
   else
      if(RST_DIR_LD)
         RESET_CNFG <= RCV_IN_REG;
      else if(RST_FL_LD)
         RESET_CNFG <= CNFG_DCD_REG[15:0];
      else if(RST_BSET)
         RESET_CNFG <= BMASK[15:0] | RESET_CNFG;
      else if(RST_BCLR)
         RESET_CNFG <= BMASK[15:0] & RESET_CNFG;
      else
         RESET_CNFG <= RESET_CNFG;
end

always @(posedge CLK or posedge RST)
begin
   if(RST)
      VME_CNFG <= `VME_Cnfg_Dflt;
   else
      if(VME_DIR_LD_H)
         VME_CNFG <= {RCV_IN_REG,VME_CNFG[15:0]};
      else if(VME_DIR_LD_L)
         VME_CNFG <= {VME_CNFG[31:16],RCV_IN_REG};
      else if(VME_FL_LD)
         VME_CNFG <= CNFG_DCD_REG[47:16];
      else if(VME_BSET)
         VME_CNFG <= BMASK | VME_CNFG;
      else if(VME_BCLR)
         VME_CNFG <= BMASK & VME_CNFG;
      else
         VME_CNFG <= VME_CNFG;
end

// VME Bus TimeOut Register
always @(posedge CLK or posedge RST)
begin
   if(RST)
      VME_BTO <= `VME_BTO_Dflt;
   else
      if(BTO_DIR_LD)
         VME_BTO <= RCV_IN_REG;
      else if(BTO_FL_LD)
         VME_BTO <= CNFG_DCD_REG[15:0];
      else
         VME_BTO <= VME_BTO;
end

// VME BusGrant TimeOut Register
always @(posedge CLK or posedge RST)
begin
   if(RST)
      VME_BGTO <= `VME_BGTO_Dflt;
   else
      if(BGTO_DIR_LD)
         VME_BGTO <= RCV_IN_REG;
      else if(BGTO_FL_LD)
         VME_BGTO <= CNFG_DCD_REG[47:32];
      else
         VME_BGTO <= VME_BGTO;
end

//////////////////////////////////////////////////////////////////
//                                                              //
// Output MUX for configuration registers                       //
//                                                              //
//////////////////////////////////////////////////////////////////

always @(posedge CLK or posedge RST)
begin
   if(RST)
      TX_OUT_REG <= 16'h0000;    // 50 usec.
   else
      if(ENA_RD_SEQ)
         case(SEL_CR)
            `Sel_Eth_CR:
               TX_OUT_REG <= ETHER_CNFG;
            `Sel_Ext_CR:
               TX_OUT_REG <= EXT_FIFO_CNFG;
            `Sel_Rst_CR:
               TX_OUT_REG <= RESET_CNFG;
            `Sel_VMEh_CR:
               TX_OUT_REG <= VME_CNFG[31:16];
            `Sel_VMEl_CR:
               TX_OUT_REG <= VME_CNFG[15:0];
            `Sel_BTO_CR:
               TX_OUT_REG <= VME_BTO;
            `Sel_BGTO_CR:
               TX_OUT_REG <= VME_BGTO;
            default:
               TX_OUT_REG <= ETHER_CNFG;
         endcase
      else if(SND_DFLT)
         TX_OUT_REG <= {11'h000,CNFG_DCD_REG[28:24]};
      else if(SND_CNFGA)
         case({S3,S2,S1})
            3'b001:
               TX_OUT_REG <= CNFG_DCD_REG[47:32];
            3'b011:
               TX_OUT_REG <= CNFG_DCD_REG[31:16];
            3'b111:
               TX_OUT_REG <= CNFG_DCD_REG[15:0];
            default:
               TX_OUT_REG <= CNFG_DCD_REG[47:32];
         endcase
      else if(SND_CNFGB)
         TX_OUT_REG <= CNFG_DCD_REG[47:32];
      else if(SND_MAC)
         case(ACNT)
            3'd0:
               TX_OUT_REG <= CNFG_DCD_REG[47:32];
            3'd1:
               TX_OUT_REG <= CNFG_DCD_REG[31:16];
            3'd2:
               TX_OUT_REG <= CNFG_DCD_REG[15:0];
            default:
               TX_OUT_REG <= CNFG_DCD_REG[47:32];
         endcase
      else
         TX_OUT_REG <= 16'h0000;
end

// Selection sequence counter
always @(posedge CLK or posedge SEL_CR_RST)
begin
   if(SEL_CR_RST)
      SEL_CR <= 3'd0;
   else
      if(ENA_RD_SEQ || ENA_WRT_SEQ)
         SEL_CR <= SEL_CR + 1;
      else
         SEL_CR <= SEL_CR;
end



// counter for one page of configuration memory //
always @(posedge CLK or posedge RST)
begin
   if(RST)
      AD_CNT <= 7'h00;
   else
      if(LD_AC)
         AD_CNT <= TX_IN_REG[6:0];
      else if(SHIFT)
         AD_CNT <= AD_CNT -1;
      else
         AD_CNT <= AD_CNT;
end

// input pipeline to collect 2 ECC pairs //
always @(posedge CLK)
begin
   if(SHIFT && FILL)
      begin
         PARITY[7:0] <= TX_IN_REG[7:0];
         PARITY[15:8] <= PARITY[7:0];
         PARITY[23:16] <= PARITY[15:8];
         DATA[7:0] <= PARITY[23:16];
         DATA[23:8] <= 16'h0000;
      end
   else if(SHIFT)
      begin
         PARITY[7:0] <= TX_IN_REG[7:0];
         PARITY[15:8] <= PARITY[7:0];
         PARITY[23:16] <= PARITY[15:8];
         DATA[7:0] <= PARITY[23:16];
         DATA[15:8] <= DATA[7:0];
         DATA[23:16] <= DATA[15:8];
      end
   else
      begin
         PARITY <= PARITY;
         DATA <= DATA;
      end
end

always @(posedge CLK or posedge RST)
begin
   if(RST)
      SIXB_PHASE <= 3'd0;
   else
      if(SHIFT)
         if(FILL || (SIXB_PHASE == 3'd5))
            SIXB_PHASE <= 3'd0;
         else
            SIXB_PHASE <= SIXB_PHASE+1;
      else
         SIXB_PHASE <= SIXB_PHASE;
end

always @(posedge CLK)
begin
   BT0_DLY <= BT0;
end

// State machine for converting FLASH format to FIFO width //
always @(posedge CLK or posedge RST)
begin: F2C_FSM
   if(RST)
      F2C_ST <= `F2C_Idle;
   else
      case(F2C_ST)
         `F2C_Idle:
            if(FL_CDV)
               F2C_ST <= `F2C_Shift;
            else
               F2C_ST <= `F2C_Idle;
         `F2C_Shift:
            if(AD_CNT == 7'h01)
               F2C_ST <= `F2C_Load_AC;
            else
               F2C_ST <= `F2C_Shift;
         `F2C_Load_AC:
            if(FL_CDV)
               F2C_ST <= `F2C_Shift;
            else if(FL_CND)
               F2C_ST <= `F2C_Complete;
            else
               F2C_ST <= `F2C_Load_AC;
         `F2C_Complete:
            F2C_ST <= `F2C_Idle;
         default:
            F2C_ST <= `F2C_Idle;
      endcase      
end

always @(F2C_ST or BT0 or BT0_DLY or FL_CDV)
begin
   case(F2C_ST)
      `F2C_Idle:
         begin
            SHIFT <= 0;
            LD_AC <= FL_CDV;
            PUSH_RAW <= 0;
            CMPLTD <= 0;
         end
      `F2C_Shift:
         begin
            SHIFT <= FL_CDV;
            LD_AC <= 0;
            PUSH_RAW <= BT0 && !BT0_DLY;
            CMPLTD <= 0;
         end
      `F2C_Load_AC:
         begin
            SHIFT <= 0;
            LD_AC <= FL_CDV;
            PUSH_RAW <= BT0 && !BT0_DLY;
            CMPLTD <= 0;
         end
      `F2C_Complete:
         begin
            SHIFT <= 0;
            LD_AC <= 0;
            PUSH_RAW <= 0;
            CMPLTD <= 1;
         end
      default:
         begin
            SHIFT <= 0;
            LD_AC <= 0;
            PUSH_RAW <= 0;
            CMPLTD <= 0;
         end
   endcase
end

//////////////////////////////////////////////////////////////////
//                                                              //
// Input FIFO for data coming from the Flash memory             //
// (FR_FL_FF : FRom_FLash_FiFo)                                 //
//                                                              //
//////////////////////////////////////////////////////////////////

parameter FR_FL_FF_p2depth = 4;
defparam FR_FL_FF.p2depth = FR_FL_FF_p2depth;
defparam FR_FL_FF.width = 48;
defparam FR_FL_FF.almostmt = 2;
defparam FR_FL_FF.almostfull = (1<<FR_FL_FF_p2depth) - 2;
fifo_dpXn_blk  FR_FL_FF(.clk(CLK),.rst(RST),.push(PUSH_RAW),.pop(READ_RAW),
                       .mt(RAW_MT),.full(RAW_FULL),.amt(RAW_AMT),.af(RAW_AF),.wterr(RAW_WTERR),.rderr(RAW_RDERR),
             .din({DATA[23:0],PARITY[23:0]}),.dout({RWD1,RWD2,RWP1,RWP2}),.cnt(RAW_CNT));

ecc_decode ecc_decode_CNFG1(.CLK(CLK),.RCV_DATA(RWD1),.RCV_PARITY(RWP1),.GOOD_WORD(VP1),
              .DCD_DATA(DCD_DATA[23:12]),.DCD_PARITY(DCD_PAR1),.BAD_TX(BTX1));
ecc_decode ecc_decode_CNFG2(.CLK(CLK),.RCV_DATA(RWD2),.RCV_PARITY(RWP2),.GOOD_WORD(VP2),
              .DCD_DATA(DCD_DATA[11:0]),.DCD_PARITY(DCD_PAR2),.BAD_TX(BTX2));

// Counter for MAC memory addresses
always @(posedge CLK or posedge ACNT_RST)
begin
   if(ACNT_RST)
      ACNT <= 2'd0;
   else
      if(ACNT_ENA)
         if(ACNT == 2'd2)
            ACNT <= 2'd0;
         else
            ACNT <= ACNT + 1;
      else
         ACNT <= ACNT;
end

// Register/counter for MAC ID (MAC type)
always @(posedge CLK or posedge RST)
begin
   if(RST)
      MAC_ID <= 4'h0;
   else
      if(LD_MAC_ID)
         MAC_ID <= RCV_IN_REG[3:0];
      else if((ALL_MAC && RST_MAC_ID) || SET_MAC_ID)
         MAC_ID <= 4'h8;
      else if(MID_ENA && ALL_MAC && (ACNT == 2'd2))
         MAC_ID <= MAC_ID + 1;
      else if(MID_INC)
         MAC_ID <= MAC_ID + 1;
      else
         MAC_ID <= MAC_ID;
end

always @(SND_MAC_ENA or MAC_ID or SND_DFLT_ENA or SND_CNFG_ENA)
begin
   if(SND_MAC_ENA)
      case(MAC_ID)
         4'h0,4'h8:
            M_OFFSET <= 6'h00;
         4'h1,4'h9:
            M_OFFSET <= 6'h0C;
         4'h2,4'hA:
            M_OFFSET <= 6'h18;
         4'h3,4'hB:
            M_OFFSET <= 6'h24;
         4'h4,4'hC:
            M_OFFSET <= 6'h30;
         default:
            M_OFFSET <= 6'h00;
      endcase
   else if(SND_DFLT_ENA)
      M_OFFSET <= 6'h3C;
   else if(SND_CNFG_ENA)
      M_OFFSET <= 6'h00;
   else
      M_OFFSET <= 6'h00;
end

always @(posedge CLK or posedge RA_RST)
begin
   if(RA_RST)
      RELADDR <= 6'h00;
   else
      if(RA_ENA || RA_ENA_DFLT || RA_ENA_CNFG)
         if(RA_ENA && (RELADDR == 6'h0B))
            RELADDR <= 6'h00;
         else
            RELADDR <= RELADDR + 1;
      else
         RELADDR <= RELADDR;
end

always @(posedge CLK)
begin
   LD1_H <= ACNT_ENA && !MAC_WE && !MAC_WE2 && (ACNT == 2'd0);
   LD1_M <= ACNT_ENA && !MAC_WE && !MAC_WE2 && (ACNT == 2'd1);
   LD1_L <= ACNT_ENA && !MAC_WE && !MAC_WE2 && (ACNT == 2'd2);
   LD2_H <= LD1_H;
   LD2_M <= LD1_M;
   LD2_L <= LD1_L;
   LD3_H <= LD2_H;
   LD3_M <= LD2_M;
   LD3_L <= LD2_L;
   LD_H <= LD3_H;
   LD_M <= LD3_M;
   LD_L <= LD3_L;
   LD_MAC_DONE <= LD3_L;
end

// Scratch register for MAC addresses
always @(posedge CLK or posedge RST)
begin
   if(RST)
      MAC_REG <= 48'h000000000000;
   else
      if(LD_H)
         MAC_REG <= {MACBUS_IN_REG,MAC_REG[31:0]};
      else if(LD_M)
         MAC_REG <= {MAC_REG[47:32],MACBUS_IN_REG,MAC_REG[15:0]};
      else if(LD_L)
         MAC_REG <= {MAC_REG[47:16],MACBUS_IN_REG};
      else
         MAC_REG <= MAC_REG;
end

// ECC Encoder Mux
always @(posedge CLK)
begin
   if(MAC1)
      begin
         ECC_MUXH <= MAC_REG[47:36];
         ECC_MUXL <= MAC_REG[36:24];
      end
   else if(MAC2)
      begin
         ECC_MUXH <= MAC_REG[23:12];
         ECC_MUXL <= MAC_REG[11:0];
      end
   else if(DFLT)
      begin
         ECC_MUXH <= 12'h000;
         ECC_MUXL <= {7'h00,CNFG_NUM};
      end
   else if(ENA_CNFG && (RND == 3'h0))//Ethernet CR
      begin
         ECC_MUXH <= ETHER_CNFG[15:4];
         ECC_MUXL <= {ETHER_CNFG[3:0],EXT_FIFO_CNFG[15:8]};
      end
   else if(ENA_CNFG && (RND == 3'h1))//External FIFO CR and Reset Enables CR
      begin
         ECC_MUXH <= {EXT_FIFO_CNFG[7:0],RESET_CNFG[15:12]};
         ECC_MUXL <= {RESET_CNFG[11:0]};
      end
   else if(ENA_CNFG && (RND == 3'h2))//VME CR
      begin
         ECC_MUXH <= VME_CNFG[31:20];
         ECC_MUXL <= VME_CNFG[19:8];
      end
   else if(ENA_CNFG && (RND == 3'h3))//VME CR and Bus TimeOut
      begin
         ECC_MUXH <= {VME_CNFG[7:0],VME_BTO[15:12]};
         ECC_MUXL <= VME_BTO[11:0];
      end
   else if(ENA_CNFG && (RND == 3'h4))//Bus Grant TimeOut
      begin
         ECC_MUXH <= VME_BGTO[15:4];
         ECC_MUXL <= {VME_BGTO[3:0],8'h00};
      end
   else
      begin
         ECC_MUXH <= 12'h000;
         ECC_MUXL <= 12'h000;
      end
end



// ECC Encoders
ecc_encode ecc_encode_CNFG1(.CLK(CLK),.DATA(ECC_MUXH),.PARITY(ECC_PARH));
ecc_encode ecc_encode_CNFG2(.CLK(CLK),.DATA(ECC_MUXL),.PARITY(ECC_PARL));

always @(posedge CLK)
begin
   ECC_MUXH1 <= ECC_MUXH;
   ECC_MUXL1 <= ECC_MUXL;
end

always @(posedge CLK or posedge RST)
begin
   if(RST)
      HDR_REG <= 16'h0000;
   else
      if(LD_HDR1 || DFLT || SC1_LD_HDR1)
         HDR_REG <= HDR1;
      else if(LD_HDR2A)
         HDR_REG <= HDR2A;
      else if(LD_HDR2B)
         HDR_REG <= HDR2B;
      else if(LD_HDR2C)
         HDR_REG <= HDR2C;
      else
         HDR_REG <= HDR_REG;
end

always @(posedge CLK)
begin
   CNFG_RCV_ODV <= DV_MAC || DV_DFLT || DV_CNFG;
end

always @(posedge CLK or posedge RST)
begin
   if(RST)
      FL_DATA <= 48'h000000000000;
   else
      if(LD_FL_DATA_MAC || LD_FL_DATA_CNFG)
         if(FL_WRT)
            FL_DATA <= {ECC_MUXH1,ECC_MUXL1,ECC_PARH,ECC_PARL};
         else
            FL_DATA <= 48'h000000000000;
      else if(LD_FL_DATA_DFLT)
         if(FL_WRT)
            FL_DATA <= {ECC_MUXL1[7:0],ECC_PARH,ECC_PARL,16'h0000};
         else
            FL_DATA <= 48'h000000000000;
      else if(SH_FL_DATA_MAC || SH_FL_DATA_DFLT || SH_FL_DATA_CNFG)
         begin
            FL_DATA[47:40] <= FL_DATA[39:32];
            FL_DATA[39:32] <= FL_DATA[31:24];
            FL_DATA[31:24] <= FL_DATA[23:16];
            FL_DATA[23:16] <= FL_DATA[15:8];
            FL_DATA[15:8] <= FL_DATA[7:0];
            FL_DATA[7:0] <= FL_DATA[7:0];
         end
      else
         FL_DATA <= FL_DATA;
end

always @(posedge CLK)
begin
   HEAD_OUT <= HEAD_OUT_MAC || HEAD_OUT_DFLT || HEAD_OUT_CNFG;
   BA_DATA_OUT <= BA_DATA_OUT_MAC ||BA_DATA_OUT_DFLT || BA_DATA_OUT_CNFG;
end

always @(posedge CLK)
begin
   if(HEAD_OUT)
      RCV_OUT_REG <= HDR_REG;
   else if(BA_DATA_OUT)
      RCV_OUT_REG <= {2'b00,BA,FL_DATA[47:40]};
   else
      RCV_OUT_REG <= 16'h0000;
end





// State machine for sending MAC address to the Flash //
always @(posedge CLK or posedge RST)
begin: SM1_FSM     // Send MAC Sequence for sending MAC info to Flash controller
   if(RST)
      SM1_ST <= `SM1_Idle;
   else
      case(SM1_ST)
         `SM1_Idle:
            if(SND_MAC_ENA)
               if((MAC_ID <= 4'h8))
                  SM1_ST <= `SM1_Hdr1;
               else
                  SM1_ST <= `SM1_NoHdr1;
            else
               SM1_ST <= `SM1_Idle;
         `SM1_Hdr1:
            SM1_ST <= `SM1_Hdr2a;
         `SM1_Hdr2a:
            SM1_ST <= `SM1_MC1_Load;
         `SM1_NoHdr1:
            SM1_ST <= `SM1_NoHdr2;
         `SM1_NoHdr2:
            SM1_ST <= `SM1_MC1_Load;
         `SM1_MC1_Load:
            SM1_ST <= `SM1_MC1_Shift;
         `SM1_MC1_Shift:
            if(HOLD3)
               SM1_ST <= `SM1_MC2_ShiftA;
            else
               SM1_ST <= `SM1_MC1_Shift;
         `SM1_MC2_ShiftA:
            if(SHIFT_DONE)
               SM1_ST <= `SM1_MC2_Load;
            else
               SM1_ST <= `SM1_MC2_ShiftA;
         `SM1_MC2_Load:
            SM1_ST <= `SM1_MC2_ShiftB;
         `SM1_MC2_ShiftB:
            if(HOLD3)
               SM1_ST <= `SM1_Shift;
            else
               SM1_ST <= `SM1_MC2_ShiftB;
         `SM1_Shift:
            if(SHIFT_DONE)
               SM1_ST <= `SM1_Done;
            else
               SM1_ST <= `SM1_Shift;
         `SM1_Done:
            SM1_ST <= `SM1_Idle;
         default:
            SM1_ST <= `SM1_Idle;
      endcase      
end


always @(SM1_ST)
begin
   case(SM1_ST)
      `SM1_Hdr1:
         begin
            LD_HDR1 <= 1;
            LD_HDR2A <= 0;
            HEAD_OUT_MAC <= 1;
            BA_DATA_OUT_MAC <= 0;
         end
      `SM1_Hdr2a:
         begin
            LD_HDR1 <= 0;
            LD_HDR2A <= 1;
            HEAD_OUT_MAC <= 1;
            BA_DATA_OUT_MAC <= 0;
         end
      default:
         begin
            LD_HDR1 <= 0;
            LD_HDR2A <= 0;
            HEAD_OUT_MAC <= 0;
            BA_DATA_OUT_MAC <= 1;
         end
   endcase
end


always @(SM1_ST)
begin
   case(SM1_ST)
      `SM1_MC2_ShiftA,`SM1_Shift:
         SM1_SD1 <= 1;
      default:
         SM1_SD1 <= 0;
   endcase
end

always @(SM1_ST)
begin
   case(SM1_ST)
      `SM1_MC1_Load,`SM1_MC2_Load:
         LD_FL_DATA_MAC <= 1;
      default:
         LD_FL_DATA_MAC <= 0;
   endcase
end

always @(SM1_ST)
begin
   case(SM1_ST)
      `SM1_MC1_Shift,`SM1_MC2_ShiftA,`SM1_MC2_ShiftB,`SM1_Shift:
         SH_FL_DATA_MAC <= 1;
      default:
         SH_FL_DATA_MAC <= 0;
   endcase
end

always @(SM1_ST)
begin
   case(SM1_ST)
      `SM1_Hdr1,`SM1_Hdr2a,`SM1_NoHdr1,`SM1_NoHdr2,`SM1_MC1_Load,`SM1_MC1_Shift:
         begin
            MAC1 <= 1;
            MAC2 <= 0;
         end
      `SM1_MC2_ShiftA,`SM1_MC2_Load,`SM1_MC2_ShiftB:
         begin
            MAC1 <= 0;
            MAC2 <= 1;
         end
      default:
         begin
            MAC1 <= 0;
            MAC2 <= 0;
         end
   endcase
end

always @(SM1_ST)
begin
   case(SM1_ST)
      `SM1_Idle,`SM1_Hdr1,`SM1_Hdr2a,`SM1_NoHdr1,`SM1_NoHdr2,`SM1_MC1_Load:
         RA_ENA <= 0;
      default:
         RA_ENA <= 1;
   endcase
end


always @(SM1_ST or FL_WRT or MAC_ID)
begin
   case(SM1_ST)
      `SM1_Idle,`SM1_Hdr1,`SM1_NoHdr1,`SM1_NoHdr2:
         DV_MAC <= 0;
      `SM1_MC1_Load:
         DV_MAC <= (MAC_ID <= 4'h8);
      default:
         DV_MAC <= 1;
   endcase
end

always @(SM1_ST)
begin
   case(SM1_ST)
      `SM1_Done:
         SND_MAC_DONE <= 1;
      default:
         SND_MAC_DONE <= 0;
   endcase
end

always @(posedge CLK)
begin
   HOLD1 <= LD_FL_DATA_MAC || LD_FL_DATA_DFLT || LD_FL_DATA_CNFG;
   HOLD2 <= HOLD1;
   HOLD3 <= HOLD2;
   SHIFT_DONE <= SM1_SD1 || SC1_SD1;
end
         
// State machine for sending Default configuration number to the Flash //
always @(posedge CLK or posedge RST)
begin: SD1_FSM     // Send Default num; Sequence for sending Default info to Flash controller
   if(RST)
      SD1_ST <= `SD1_Idle;
   else
      case(SD1_ST)
         `SD1_Idle:
            if(SND_DFLT_ENA)
               SD1_ST <= `SD1_Hdr1;
            else
               SD1_ST <= `SD1_Idle;
         `SD1_Hdr1:
            SD1_ST <= `SD1_Hdr2b;
         `SD1_Hdr2b:
            SD1_ST <= `SD1_Dflt_Load;
         `SD1_Dflt_Load:
            SD1_ST <= `SD1_Dflt_Shift;
         `SD1_Dflt_Shift:
            if(HOLD3)
               SD1_ST <= `SD1_Done;
            else
               SD1_ST <= `SD1_Dflt_Shift;
         `SD1_Done:
            SD1_ST <= `SD1_Idle;
         default:
            SD1_ST <= `SD1_Idle;
      endcase      
end


always @(SD1_ST)
begin
   case(SD1_ST)
      `SD1_Hdr1:
         begin
            DFLT <= 1;
            LD_HDR2B <= 0;
            HEAD_OUT_DFLT <= 1;
            BA_DATA_OUT_DFLT <= 0;
         end
      `SD1_Hdr2b:
         begin
            DFLT <= 0;
            LD_HDR2B <= 1;
            HEAD_OUT_DFLT <= 1;
            BA_DATA_OUT_DFLT <= 0;
         end
      default:
         begin
            DFLT <= 0;
            LD_HDR2B <= 0;
            HEAD_OUT_DFLT <= 0;
            BA_DATA_OUT_DFLT <= 1;
         end
   endcase
end
always @(SD1_ST)
begin
   case(SD1_ST)
      `SD1_Dflt_Load:
         LD_FL_DATA_DFLT <= 1;
      default:
         LD_FL_DATA_DFLT <= 0;
   endcase
end

always @(SD1_ST)
begin
   case(SD1_ST)
      `SD1_Dflt_Shift:
         SH_FL_DATA_DFLT <= 1;
      default:
         SH_FL_DATA_DFLT <= 0;
   endcase
end
always @(SD1_ST)
begin
   case(SD1_ST)
      `SD1_Dflt_Shift,`SD1_Done:
         RA_ENA_DFLT <= 1;
      default:
         RA_ENA_DFLT <= 0;
   endcase
end


always @(SD1_ST)
begin
   case(SD1_ST)
      `SD1_Idle,`SD1_Hdr1:
         DV_DFLT <= 0;
      default:
         DV_DFLT <= 1;
   endcase
end
always @(SD1_ST)
begin
   case(SD1_ST)
      `SD1_Done:
         SND_DFLT_DONE <= 1;
      default:
         SND_DFLT_DONE <= 0;
   endcase
end

// ROM for Config Num to Start Page Map
always @(posedge CLK)
begin: Strt_Pg_ROM
   case(CNFG_NUM)
      5'd00: ST_PG <= 7'h01;  //   1
      5'd01: ST_PG <= 7'h07;  //   7
      5'd02: ST_PG <= 7'h0D;  //  13
      5'd03: ST_PG <= 7'h13;  //  19
      5'd04: ST_PG <= 7'h19;  //  25
      5'd05: ST_PG <= 7'h1F;  //  31
      5'd06: ST_PG <= 7'h25;  //  37
      5'd07: ST_PG <= 7'h2B;  //  43
      5'd08: ST_PG <= 7'h31;  //  49
      5'd09: ST_PG <= 7'h37;  //  55
      5'd10: ST_PG <= 7'h3D;  //  61
      5'd11: ST_PG <= 7'h43;  //  67
      5'd12: ST_PG <= 7'h49;  //  73
      5'd13: ST_PG <= 7'h4F;  //  79
      5'd14: ST_PG <= 7'h55;  //  85
      5'd15: ST_PG <= 7'h5B;  //  91
      5'd16: ST_PG <= 7'h61;  //  97
      5'd17: ST_PG <= 7'h67;  // 103
      5'd18: ST_PG <= 7'h6D;  // 109
      5'd19: ST_PG <= 7'h73;  // 115
      5'd20: ST_PG <= 7'h79;  // 121
      default: ST_PG <= 7'h7F;
   endcase
end

// State machine for sending Configuration Registers to the Flash //
always @(posedge CLK or posedge RST)
begin: SC1_FSM     // Send Cnfg; Sequence for sending Configuration info to Flash controller
   if(RST)
      SC1_ST <= `SC1_Idle;
   else
      case(SC1_ST)
         `SC1_Idle:
            if(SND_CNFG_ENA)
               SC1_ST <= `SC1_Hdr1;
            else
               SC1_ST <= `SC1_Idle;
         `SC1_Hdr1:
            SC1_ST <= `SC1_Hdr2c;
         `SC1_Hdr2c:
            SC1_ST <= `SC1_Cnfg_LoadA;
         `SC1_Cnfg_LoadA:
            SC1_ST <= `SC1_Cnfg_ShiftA;
         `SC1_Cnfg_ShiftA:
            if(HOLD3)
               if(RND == 4)
                  SC1_ST <= `SC1_Cnfg_ShiftC;
               else
                  SC1_ST <= `SC1_Cnfg_ShiftB;
            else
               SC1_ST <= `SC1_Cnfg_ShiftA;
         `SC1_Cnfg_ShiftB:
            if(SHIFT_DONE)
               SC1_ST <= `SC1_Cnfg_LoadB;
            else
               SC1_ST <= `SC1_Cnfg_ShiftB;
         `SC1_Cnfg_LoadB:
            SC1_ST <= `SC1_Cnfg_ShiftA;
         `SC1_Cnfg_ShiftC:
            if(SHIFT_DONE)
               SC1_ST <= `SC1_Done;
            else
               SC1_ST <= `SC1_Cnfg_ShiftC;
         `SC1_Done:
            SC1_ST <= `SC1_Idle;
         default:
            SC1_ST <= `SC1_Idle;
      endcase      
end


always @(SC1_ST)
begin
   case(SC1_ST)
      `SC1_Hdr1:
         begin
            SC1_LD_HDR1 <= 1;
            LD_HDR2C <= 0;
            HEAD_OUT_CNFG <= 1;
            BA_DATA_OUT_CNFG <= 0;
         end
      `SC1_Hdr2c:
         begin
            SC1_LD_HDR1 <= 0;
            LD_HDR2C <= 1;
            HEAD_OUT_CNFG <= 1;
            BA_DATA_OUT_CNFG <= 0;
         end
      default:
         begin
            SC1_LD_HDR1 <= 0;
            LD_HDR2C <= 0;
            HEAD_OUT_CNFG <= 0;
            BA_DATA_OUT_CNFG <= 1;
         end
   endcase
end

always @(SC1_ST)
begin
   case(SC1_ST)
      `SC1_Cnfg_ShiftB,`SC1_Cnfg_ShiftC:
         SC1_SD1 <= 1;
      default:
         SC1_SD1 <= 0;
   endcase
end
always @(SC1_ST or HOLD3)
begin
   case(SC1_ST)
      `SC1_Cnfg_ShiftA:
         INC_RND <= HOLD3;
      default:
         INC_RND <= 0;
   endcase
end
always @(SC1_ST)
begin
   case(SC1_ST)
      `SC1_Cnfg_LoadA,`SC1_Cnfg_LoadB:
         LD_FL_DATA_CNFG <= 1;
      default:
         LD_FL_DATA_CNFG <= 0;
   endcase
end

always @(SC1_ST)
begin
   case(SC1_ST)
      `SC1_Cnfg_ShiftA,`SC1_Cnfg_ShiftB,`SC1_Cnfg_ShiftC:
         SH_FL_DATA_CNFG <= 1;
      default:
         SH_FL_DATA_CNFG <= 0;
   endcase
end

always @(SC1_ST)
begin
   case(SC1_ST)
      `SC1_Idle,`SC1_Cnfg_ShiftC,`SC1_Done:
         ENA_CNFG <= 0;
      default:
         ENA_CNFG <= 1;
   endcase
end
always @(SC1_ST)
begin
   case(SC1_ST)
      `SC1_Cnfg_ShiftA,`SC1_Cnfg_ShiftB,`SC1_Cnfg_ShiftC,`SC1_Cnfg_LoadB,`SC1_Done:
         RA_ENA_CNFG <= 1;
      default:
         RA_ENA_CNFG <= 0;
   endcase
end


always @(SC1_ST)
begin
   case(SC1_ST)
      `SC1_Idle,`SC1_Hdr1:
         DV_CNFG <= 0;
      default:
         DV_CNFG <= 1;
   endcase
end
always @(SC1_ST)
begin
   case(SC1_ST)
      `SC1_Done:
         SND_CNFG_DONE <= 1;
      default:
         SND_CNFG_DONE <= 0;
   endcase
end
always @(SC1_ST)
begin
   case(SC1_ST)
      `SC1_Idle:
         RST_RND <= 1;
      default:
         RST_RND <= 0;
   endcase
end
always @(posedge CLK or posedge RND_RST)
begin
   if(RND_RST)
      RND <= 3'h0;
   else
      if(INC_RND)
         RND <= RND + 1;
      else
         RND <= RND;
end



//////////////////////////////////////////////////////////////
//                                                          //
// ROM for initializing MAC memories when there is no       //
// valid default configuration in the Flash memory          //
//                                                          //
//////////////////////////////////////////////////////////////

always @(posedge CLK)
begin: MAC_defaults
   case(MAC_INI_CNT)
      4'h0: MAC_INI <= `Dflt_MAC0;
      4'h1: MAC_INI <= `Dflt_MAC1;
      4'h2: MAC_INI <= `Dflt_MAC2;
      4'h3: MAC_INI <= `Dflt_MAC3;
      4'h4: MAC_INI <= `Dflt_MAC4;
      4'h5: MAC_INI <= `Dflt_MAC5;
      4'h6: MAC_INI <= `Dflt_MAC6;
      4'h7: MAC_INI <= `Dflt_MAC7;
      4'h8: MAC_INI <= `Dflt_MAC8;
      4'h9: MAC_INI <= `Dflt_MAC9;
      4'hA: MAC_INI <= `Dflt_MACA;
      4'hB: MAC_INI <= `Dflt_MACB;
      4'hC: MAC_INI <= `Dflt_MACC;
      4'hD: MAC_INI <= `Dflt_MACD;
      4'hE: MAC_INI <= `Dflt_MACE;
      4'hF: MAC_INI <= 21'h12FFFF;
      default: MAC_INI <= 21'h12FFFF;
   endcase
end

always @(posedge CLK or posedge RST)
begin
   if(RST)
      MAC_INI_CNT <= 4'h0;
   else
      if(HRDW_INI && HRDW_INI_DONE)
         MAC_INI_CNT <= 4'h0;
      else if(HRDW_INI)
         MAC_INI_CNT <= MAC_INI_CNT + 1;
      else 
         MAC_INI_CNT <= MAC_INI_CNT;
end


endmodule
