// Verilog model

`timescale 1ns / 1ps

// synthesis attribute ram_style of ether_ctrl is distributed
// synthesis attribute rom_style of ether_ctrl is distributed

module ether_ctrl(CLK,POW_ON_RST,SW2L_BIT,ETHER_CNFG,TX_ERR,TXSTATUS,RX_ERR,RXSTATUS,
                  TXFORCECRCERR,TXINHIBIT,TDIS,TX_INPRGS,SER_LPBCK,PAR_LPBCK,
                  PASS_THRU,PROMISC,PROT_ENA,XTEND,SPONT_PKT_ENA,PRIO_PKT_ENA,NORM_PKT_ENA);

    input CLK;
    input POW_ON_RST;
    input [3:0] SW2L_BIT;
    input [15:0] ETHER_CNFG;
    input TX_ERR;
    input [15:0] TXSTATUS;
    input RX_ERR;
    input [15:0] RXSTATUS;
    output TXFORCECRCERR;
    output TXINHIBIT;
    output TDIS;
    output TX_INPRGS;
    output SER_LPBCK;
    output PAR_LPBCK;
    output PASS_THRU;
    output PROMISC;
    output PROT_ENA;
    output XTEND;
    output SPONT_PKT_ENA;
    output PRIO_PKT_ENA;
    output NORM_PKT_ENA;


reg PASS_THRU;
reg PROMISC;
reg PROT_ENA;
reg XTEND;

wire [3:0] SW2;
wire TX_INPRGS;
wire TDIS;
wire SPONT_PKT_ENA;
wire PRIO_PKT_ENA;
wire NORM_PKT_ENA;
wire SWITCH_CNTRL;


`ifdef HEADER
`else
`include "header.v"
`endif



assign TXINHIBIT = ETHER_CNFG[0];
assign TDIS = ETHER_CNFG[1];
assign SPONT_PKT_ENA = ETHER_CNFG[6];
assign PRIO_PKT_ENA = 1;
assign NORM_PKT_ENA = 1;
assign SWITCH_CNTRL = ETHER_CNFG[7];
assign SW2 = ~SW2L_BIT;
assign TXFORCECRCERR = 0;
assign SER_LPBCK = 0;
assign PAR_LPBCK = 0;
assign TX_INPRGS = !(TXSTATUS[4] && TXSTATUS[0] && !TXSTATUS[8]);



always @(SW2 or SWITCH_CNTRL or ETHER_CNFG)
begin
   if(SWITCH_CNTRL)
      case (SW2)
         4'h0: // Protocol Enabled on transmitted packets; no Pass-Thru; not Promiscuous; no Carrier Extend                              
            begin
               PASS_THRU <= 0;
               PROMISC <= 0;
               PROT_ENA <= 1;
               XTEND <= 0;
            end
         4'h1: // No Protocol on transmitted packets                              
            begin                              
               PASS_THRU <= 0;
               PROMISC <= 0;
               PROT_ENA <= 0;
               XTEND <= 0;
            end
         4'h2: // Pass-thru packets as is; not Promiscuous; 
            begin
               PASS_THRU <= 1;
               PROMISC <= 0;
               PROT_ENA <= 0;
               XTEND <= 0;
            end
         4'h3: // Pass-thru any packet as is;
            begin
               PASS_THRU <= 1;
               PROMISC <= 1;
               PROT_ENA <= 0;
               XTEND <= 0;
            end
         4'h4: // Promiscuous mode with protocol enabled
            begin
               PASS_THRU <= 0;
               PROMISC <= 1;
               PROT_ENA <= 1;
               XTEND <= 0;
            end
         4'h5: // Carrier Extend with Protocol enabled
            begin
               PASS_THRU <= 0;
               PROMISC <= 0;
               PROT_ENA <= 1;
               XTEND <= 1;
            end
         default: // Protocol enabled
            begin
               PASS_THRU <= 0;
               PROMISC <= 0;
               PROT_ENA <= 1;
               XTEND <= 0;
            end
      endcase
   else
      begin
         PASS_THRU <= ETHER_CNFG[2];
         PROMISC   <= ETHER_CNFG[3];
         PROT_ENA  <= ETHER_CNFG[4];
         XTEND     <= ETHER_CNFG[5];
      end
end

endmodule
