// Verilog model  - Wed Feb 25 15:56:24 2004

`timescale 1ns / 1ps

module pb_ctrl(CLK, TXRESET, PB_ENA, FORCE_SEND, PUSH, POP, 
               DATA_LEN, PKT_LEN, DW_ADDR, DR_ADDR, LT_CNT, TXF_CNT,
               ENB_PUSH, SEND, MT, AF, FULL,
               LP, LPOP, FRG_INC, FRG_RST, LC_RST, FRGMNT, NW_PKT,
               LT_PUSH, LT_POP, LT_LOAD, LT_CE, LT_DIR,
               TXFC_CE, TXFC_DIR, WRT_RST, RD_RST);

    input CLK;
    input TXRESET;
    input PB_ENA;
    input FORCE_SEND;
    input PUSH;
    input POP;
    input [12:0] DATA_LEN;
    input [12:0] PKT_LEN;
    input [12:0] DW_ADDR;
    input [12:0] DR_ADDR;
    input [8:0] LT_CNT;
    input [12:0] TXF_CNT;
   output ENB_PUSH;
   output SEND;
   output MT;
   output AF;
   output FULL;
   output LP;  
   output LPOP;
   output FRG_INC;  
   output FRG_RST;  
   output LC_RST;  
   output FRGMNT;  
   output NW_PKT;  
   output LT_PUSH;  
   output LT_POP;  
   output LT_LOAD;  
   output LT_CE;  
   output LT_DIR;  
   output TXFC_CE;  
   output TXFC_DIR;  
   output WRT_RST;  
   output RD_RST;  


reg L_PUSH;
reg L_POP;
reg FRC_SND;
reg LT_PUSH;
reg LT_POP;
reg LT_LOAD;
reg LT_CE;
reg LT_DIR;
reg TXFC_CE;
reg TXFC_DIR;
reg [1:0] TX_RD_ST;
reg [2:0] TX_WRT_ST;
reg SEND;
reg NW_PKT;
reg FRGMNT;
reg FRG_INC;
reg FRG_RST;
reg LC_RST;

wire ENB_PUSH;
wire LT_MT;
wire LP;
wire LPOP;
wire MAX_SEND;
wire WRT_RST;
wire RD_RST;

wire [12:0] DW_ADDR,DR_ADDR;
wire [12:0] PKT_LEN;
wire [8:0] LT_CNT;
wire [12:0] TXF_CNT;

`ifdef HEADER
`else
`include "header.v"
`endif


assign  ENB_PUSH = PUSH && PB_ENA;

assign   MT   = (TXF_CNT == `Zero_1);
assign   AF   = (TXF_CNT >= `AF_Cnt);
assign   FULL = (TXF_CNT == `Full_Cnt);

assign   LT_MT = (LT_CNT == `Zero_2);
assign   LP    = (L_PUSH && !FULL) ? 1'b1 : 1'b0;
assign   LPOP  = (L_POP && !MT) ? 1'b1 : 1'b0;

assign   MAX_SEND = (PKT_LEN == `Max_Payload);
//assign   WRT_RST  = TXRESET || ((DW_ADDR == `Term_Cnt) && LP);
//assign   RD_RST   = TXRESET || ((DR_ADDR == `Term_Cnt) && LPOP);
assign   WRT_RST  = TXRESET;      //Now using 8k FIFO's so don't need Term_Cnt
assign   RD_RST   = TXRESET;

always @(posedge CLK)
begin
   L_PUSH <= ENB_PUSH;
   L_POP  <= POP;
   FRC_SND  <= FORCE_SEND;
end


always @ (LT_PUSH or LT_POP or LT_MT)
begin
  casex ({LT_MT,LT_PUSH,LT_POP})
     3'b001:
       begin                  // count down
         LT_CE <= 1'b1;
         LT_DIR <= 1'b0;
       end
     3'bx10,3'b111:
       begin
         LT_CE <= 1'b1;          // count up
         LT_DIR <= 1'b1;
       end
     default:                  // hold
       begin
         LT_CE <= 1'b0;
         LT_DIR <= 1'b1;
       end
  endcase
end



always @ (LP or LPOP or MT or FULL)
begin
  casex ({FULL,LP,MT,LPOP})
     4'bx001,4'b1101:
       begin                     // count down
         TXFC_CE <= 1'b1;
         TXFC_DIR <= 1'b0;
       end
     4'b01x0,4'b0111:
       begin                     // count up
         TXFC_CE <= 1'b1;     
         TXFC_DIR <= 1'b1;
       end
     default:                    // hold
       begin
         TXFC_CE <= 1'b0;
         TXFC_DIR <= 1'b1;
       end
  endcase
end


    always @ (posedge CLK or posedge TXRESET)
    begin: TX_FIFO_Read_control
       if (TXRESET)
          TX_RD_ST <= `Wait_for_Payload;
       else
          case (TX_RD_ST)
             `Wait_for_Payload:
                if (LT_MT)
                   TX_RD_ST <= `Wait_for_Payload;
                else
                   TX_RD_ST <= `Load_Data_Len;
             `Load_Data_Len:
                TX_RD_ST <= `Issue_Send; 
             `Issue_Send:
                if ((DATA_LEN == `Zero_1) || (MT))
                   TX_RD_ST <= `Wait_for_Payload;
                else
                   TX_RD_ST <= `Issue_Send;
             default 
                TX_RD_ST <= `Wait_for_Payload;
          endcase
    end

    always @ (TX_RD_ST or LT_MT or DATA_LEN or MT)
    begin  
       case (TX_RD_ST) 
          `Wait_for_Payload:
             begin
                LT_POP <= (LT_MT) ? 1'b0 : 1'b1;
                LT_LOAD <= 1'b0;
                SEND <= 1'b0;
             end
          `Load_Data_Len:
             begin
                LT_POP <= 1'b0;
                LT_LOAD <= 1'b1;
                SEND <= 1'b0;
             end
          `Issue_Send:
             begin
                LT_POP <= 1'b0;
                LT_LOAD <= 1'b0;
                SEND <= ((DATA_LEN == `Zero_1) || (MT) ? 1'b0 : 1'b1);
             end
          default:
             begin
                LT_POP <= 1'b0;
                LT_LOAD <= 1'b0;
                SEND <= ((DATA_LEN == `Zero_1) || (MT) ? 1'b0 : 1'b1);
             end
      endcase
    end 
  
    always @ (posedge CLK or posedge TXRESET)
    begin: TX_FIFO_write_control
       if (TXRESET)
          TX_WRT_ST <= `Standby_1;
       else
          case (TX_WRT_ST)
             `Standby_1:
                if (FORCE_SEND && PB_ENA)
                   TX_WRT_ST <= `Push_New;
                else if (MAX_SEND && PB_ENA)
                   TX_WRT_ST <= `Push_Frag_New;
                else
                   TX_WRT_ST <= `Standby_1;
             `Push_New:
                TX_WRT_ST <= `Standby_1; 
             `Push_Frag_New:
                TX_WRT_ST <= `Standby_2; 
             `Standby_2:
                if (FORCE_SEND || FRC_SND)
                   TX_WRT_ST <= `Push_Frag_1;
                else if (MAX_SEND)
                   TX_WRT_ST <= `Push_Frag_2;
                else
                   TX_WRT_ST <= `Standby_2;
             `Push_Frag_1:
                TX_WRT_ST <= `Standby_1; 
             `Push_Frag_2:
                TX_WRT_ST <= `Standby_2; 
             default 
                TX_WRT_ST <= `Standby_1;
          endcase
    end

    always @ (TX_WRT_ST or TXRESET or MAX_SEND or FORCE_SEND or FRC_SND)
    begin  
       case (TX_WRT_ST) 
          `Standby_1:
             begin
                LT_PUSH <= 1'b0;
                NW_PKT  <= 1'b0;
                FRGMNT  <= 1'b0;
                FRG_INC <= 1'b0;
                FRG_RST <= 1'b1;
                LC_RST  <= TXRESET || MAX_SEND || FORCE_SEND || FRC_SND;
             end
          `Push_New:
             begin
                LT_PUSH <= 1'b1;
                NW_PKT  <= 1'b1;
                FRGMNT  <= 1'b0;
                FRG_INC <= 1'b0;
                FRG_RST <= 1'b0;
                LC_RST  <= 1'b0;
             end
          `Push_Frag_New:
             begin
                LT_PUSH <= 1'b1;
                NW_PKT  <= 1'b1;
                FRGMNT  <= 1'b1;
                FRG_INC <= 1'b1;
                FRG_RST <= 1'b0;
                LC_RST  <= 1'b0;
             end
          `Standby_2:
             begin
                LT_PUSH <= 1'b0;
                NW_PKT  <= 1'b0;
                FRGMNT  <= 1'b0;
                FRG_INC <= 1'b0;
                FRG_RST <= 1'b0;
                LC_RST  <= TXRESET || MAX_SEND || FORCE_SEND || FRC_SND;
             end
          `Push_Frag_1:
             begin
                LT_PUSH <= 1'b1;
                NW_PKT  <= 1'b0;
                FRGMNT  <= 1'b1;
                FRG_INC <= 1'b0;
                FRG_RST <= 1'b0;
                LC_RST  <= 1'b0;
             end
          `Push_Frag_2:
             begin
                LT_PUSH <= 1'b1;
                NW_PKT  <= 1'b0;
                FRGMNT  <= 1'b1;
                FRG_INC <= 1'b1;
                FRG_RST <= 1'b0;
                LC_RST  <= 1'b0;
             end
          default 
             begin
                LT_PUSH <= 1'b0;
                NW_PKT  <= 1'b0;
                FRGMNT  <= 1'b0;
                FRG_INC <= 1'b0;
                FRG_RST <= 1'b1;
                LC_RST  <= 1'b0;
             end
      endcase
    end 
  
endmodule
