// Verilog model  - Wed Oct 13 07:39:12 2004

`timescale 1ns / 1ps

module vme_arbiter(SYSCLK, USRCLK, ARB_ENA, ARB_TYP, BBSY, BR, SYSRESET,
      SYS_CTRL_ENA, TIME_OUT, BCLR, BG_ERR, BGIN);

    input SYSCLK;
    input USRCLK;
    input ARB_ENA;
    input [1:0] ARB_TYP;
    input BBSY;
    input [3:0] BR;
    input SYSRESET;
    input SYS_CTRL_ENA;
    input [15:0] TIME_OUT;
   output BCLR;
   output BG_ERR;
   output [3:0] BGIN;
   
wire ENA;
wire LEAD_EDG_BBSY;
wire TRAIL_EDG_BBSY;
wire BR_TM_OUT;
wire TMR_RST;

reg DB0,DB1,DB2,DB3,DB4;   
reg DB5,DB6,DB7,DB8,DB9;
reg DLY_BBSY_40NS;
reg [1:0] RRS_ST;
reg [2:0] PRI_ST;
reg [3:0] CURR_BUS;
reg [3:0] BGIN;
reg [15:0] BG_TMR;
reg BG_ERR;
reg BCLR;
   
`ifdef HEADER
`else
`include "header.v"
`endif

assign ENA = BBSY | !DLY_BBSY_40NS;
assign LEAD_EDG_BBSY = BBSY & !DB0;
assign TRAIL_EDG_BBSY = !BBSY & DB0;
assign BR_TM_OUT = (BG_TMR == TIME_OUT);
assign TMR_RST = SYSRESET || BG_ERR;

// Bus Grant Timer
always @(posedge USRCLK or posedge TMR_RST)
begin
   if(TMR_RST)
      BG_TMR <= 16'h0000;
   else
      if(|BGIN && !BBSY)
         BG_TMR <= BG_TMR + 1;
      else
         BG_TMR <= BG_TMR;   
end

//BBSY delay for creating the disable region.
always @(posedge SYSCLK)
begin
   DB0 <= BBSY;
   DB1 <= DB0;   
   DB2 <= DB1;   
   DB3 <= DB2;   
   DB4 <= DB3;   
   DB5 <= DB4;   
   DB6 <= DB5;   
   DB7 <= DB6;   
   DB8 <= DB7;   
   DB9 <= DB8;
   DLY_BBSY_40NS <= DB9;
end


//State machine for Prioritized Arbitration.
always @(posedge SYSCLK or posedge SYSRESET)
begin: VME_PRIO_ARB_FSM
   if(SYSRESET)
      PRI_ST <= `Mon_BR;
   else 
      case(PRI_ST)
         `Mon_BR:
            begin
               if(!BBSY && ENA)
                  casex (BR)
                     4'b1xxx:
                        PRI_ST <= `BR_lvl3;
                     4'b01xx:
                        PRI_ST <= `BR_lvl2;
                     4'b001x:
                        PRI_ST <= `BR_lvl1;
                     4'b0001:
                        PRI_ST <= `BR_lvl0;
                     default:
                        PRI_ST <= `Mon_BR;
                  endcase
               else
                  PRI_ST <= `Mon_BR;
            end
         `BR_lvl3:
            begin
               if(BBSY)
                  PRI_ST <= `Mon_BR;
               else if(BR_TM_OUT && !BBSY)
                  PRI_ST <= `BGrant_Err;
               else
                  PRI_ST <= `BR_lvl3;
            end
         `BR_lvl2:
            begin
               if(BBSY)
                  PRI_ST <= `Mon_BR;
               else if(BR_TM_OUT && !BBSY)
                  PRI_ST <= `BGrant_Err;
               else
                  PRI_ST <= `BR_lvl2;
            end
         `BR_lvl1:
            begin
               if(BBSY)
                  PRI_ST <= `Mon_BR;
               else if(BR_TM_OUT && !BBSY)
                  PRI_ST <= `BGrant_Err;
               else
                  PRI_ST <= `BR_lvl1;
            end
         `BR_lvl0:
            begin
               if(BBSY)
                  PRI_ST <= `Mon_BR;
               else if(BR_TM_OUT && !BBSY)
                  PRI_ST <= `BGrant_Err;
               else
                  PRI_ST <= `BR_lvl0;
            end
         `BGrant_Err:
            PRI_ST <= `Mon_BR;
         default:
            PRI_ST <= `Mon_BR;
      endcase
end

always @(PRI_ST)
begin
   case(PRI_ST)
      `BGrant_Err:
         BG_ERR <= 1;
      default:
         BG_ERR <= 0;
   endcase
end 

//State machine for Round Robin Select Arbitration.
always @(posedge SYSCLK or posedge SYSRESET)
begin: VME_RROB_ARB_FSM
   if(SYSRESET)
      RRS_ST <= `RRS0;
   else 
      case(RRS_ST)
         `RRS0:
            begin
               if(!BR[3] && !BBSY)
                  RRS_ST <= `RRS1;
               else
                  RRS_ST <= `RRS0;
            end
         `RRS1:
            begin
               if(!BR[2] && !BBSY)
                  RRS_ST <= `RRS2;
               else
                  RRS_ST <= `RRS1;
            end
         `RRS2:
            begin
               if(!BR[1] && !BBSY)
                  RRS_ST <= `RRS3;
               else
                  RRS_ST <= `RRS2;
            end
         `RRS3:
            begin
               if(!BR[0] && !BBSY)
                  RRS_ST <= `RRS0;
               else
                  RRS_ST <= `RRS3;
            end
         default:
            RRS_ST <= `RRS0;
      endcase
end


// Hold current Bus Grant line while the bus is busy.  
always @(posedge SYSCLK or posedge SYSRESET)
begin
   if(SYSRESET)
      CURR_BUS <= 4'h0;
   else
      if(LEAD_EDG_BBSY)
         CURR_BUS <= BGIN;
      else if(TRAIL_EDG_BBSY)
         CURR_BUS <= 4'h0;
      else
         CURR_BUS <= CURR_BUS;
end

// Logic for BG(3:0)IN
always @(posedge SYSCLK or posedge SYSRESET)
begin
   if(SYSRESET)
      BGIN <= 4'h0;
   else 
      if(ARB_ENA && SYS_CTRL_ENA)
         case (ARB_TYP)
            `No_Arb:        // disabled
               BGIN <= 4'h0;
            `SGL:        // Single Level Arbiter (SGL)
               begin
                  BGIN[2:0] <= 3'h0;
                  BGIN[3] <= BR[3] & !BBSY & ENA;
               end
            `PRI:        // Prioritized Arbiter (PRI)
               case (PRI_ST)
                  `Mon_BR:
                     BGIN <= 4'h0;
                  `BR_lvl3:
                     begin
                        BGIN[3] <= 1;
                        BGIN[2] <= 0;
                        BGIN[1] <= 0;
                        BGIN[0] <= 0;
                     end
                  `BR_lvl2:
                     begin
                        BGIN[3] <= 0;
                        BGIN[2] <= 1;
                        BGIN[1] <= 0;
                        BGIN[0] <= 0;
                     end
                  `BR_lvl1:
                     begin
                        BGIN[3] <= 0;
                        BGIN[2] <= 0;
                        BGIN[1] <= 1;
                        BGIN[0] <= 0;
                     end
                  `BR_lvl0:
                     begin
                        BGIN[3] <= 0;
                        BGIN[2] <= 0;
                        BGIN[1] <= 0;
                        BGIN[0] <= 1;
                     end
                  `BGrant_Err:
                     BGIN <= 4'h0;
                  default:
                     BGIN <= 4'h0;
               endcase
            `RRS:        // Round-Robin-Select Arbiter (RRS)
               case (RRS_ST)
                  `RRS0:
                     begin
                        BGIN[3] <= BR[3] & !BBSY & ENA;
                        BGIN[2] <= 0;
                        BGIN[1] <= 0;
                        BGIN[0] <= 0;
                     end
                  `RRS1:
                     begin
                        BGIN[3] <= 0;
                        BGIN[2] <= BR[2] & !BBSY & ENA;
                        BGIN[1] <= 0;
                        BGIN[0] <= 0;
                     end
                  `RRS2:
                     begin
                        BGIN[3] <= 0;
                        BGIN[2] <= 0;
                        BGIN[1] <= BR[1] & !BBSY & ENA;
                        BGIN[0] <= 0;
                     end
                  `RRS3:
                     begin
                        BGIN[3] <= 0;
                        BGIN[2] <= 0;
                        BGIN[1] <= 0;
                        BGIN[0] <= BR[0] & !BBSY & ENA;
                     end
                  default:
                     BGIN <= 4'h0;
               endcase
            default:
               BGIN <= 4'h0;
         endcase
      else
         BGIN <= 4'h0;
end

// Logic for BCLR

always @(posedge SYSCLK or posedge SYSRESET)
begin
   if(SYSRESET)
      BCLR <= 0;
   else 
      if(ARB_ENA && SYS_CTRL_ENA)
         begin
            case (ARB_TYP)
               `PRI:        // Prioritized Arbiter (PRI)
                  begin
                     if(BBSY && DB1)
                        if(BR > CURR_BUS)
                           BCLR <= 1;
                        else
                           BCLR <= 0;
                     else
                        BCLR <= 0;
                  end
               `RRS:        // Round-Robin-Select Arbiter (RRS)
                  begin
                     if(CURR_BUS == 4'h0)
                        BCLR <= 0;
                     else
                        BCLR <= |(BR&~CURR_BUS);
                  end
               default:
                  BCLR <= 0;
            endcase
         end
      else
         BCLR <= 0;
end
   
endmodule
