// Verilog model  - Thu Oct 14 07:47:07 2004

`timescale 1ns / 1ps

module vme_requester(BUS_REQ, ENA, MON_BBSY, MON_BGIN, MON_BR, MON_SYSRESET, RST,
      REQ_LVL, REQ_TYP, SYSCLK, BBSY, BGOUT, BR, BUS_GRANTED);

    input BUS_REQ;
    input ENA;
    input MON_BBSY;
    input [3:0] MON_BGIN;
    input [3:0] MON_BR;
    input MON_SYSRESET;
    input RST;
    input [1:0] REQ_LVL;
    input [1:0] REQ_TYP;
    input SYSCLK;
   output BBSY;
   output [3:0] BGOUT;
   output [3:0] BR;
   output BUS_GRANTED;
//	output RQR_TST;

wire FAIR_ENA;
wire RDY_4_REL;

reg BBSY;
reg [3:0] BGOUT;
reg [3:0] BR;
reg BUS_GRANTED;
reg LOCAL_GRANT;
reg [9:0] DMBR;
reg MY_BR;
reg MY_BGIN;
reg MY_BGOUT;
reg DRV_BGOUT;
reg S_BUS_REQ;
reg L_BUS_REQ;
reg S_BGIN;
reg D_BGIN;
reg D_BUS_REQ;
wire BBSY_HOLD_90;
wire BR_REL_D30;
reg SET_BR;
reg RELEASE_BBSY;
reg D32_BR;
reg D32_BBSY;
reg D92_BBSY;
reg DR0,DR1,DR2,DR3,DR4,DR5,DR6,DR7;
reg DB0,DB1,DB2,DB3,DB4,DB5,DB6,DB7;
reg DH0,DH1,DH2,DH3,DH4,DH5;
reg DH6,DH7,DH8,DH9,DH10,DH11;
reg DH12,DH13,DH14,DH15,DH16,DH17;
reg DH18,DH19,DH20,DH21,DH22;
//reg [13:0] RQR_TST;
   
`ifdef HEADER
`else
`include "header.v"
`endif

//always @(posedge SYSCLK)
//begin
//   RQR_TST[0] <= BBSY;
//   RQR_TST[1] <= RDY_4_REL;
//   RQR_TST[2] <= BBSY_HOLD_90;
//   RQR_TST[3] <= BR_REL_D30;
//   RQR_TST[4] <= RELEASE_BBSY;
//   RQR_TST[5] <= LOCAL_GRANT;
//   RQR_TST[6] <= DRV_BGOUT;
//   RQR_TST[7] <= S_BUS_REQ;
//   RQR_TST[8] <= D_BUS_REQ;
//   RQR_TST[9] <= L_BUS_REQ;
//   RQR_TST[10] <= SET_BR;
//   RQR_TST[11] <= BR[3];
//   RQR_TST[12] <= S_BGIN;
//   RQR_TST[13] <= D_BGIN;
//end


always @(REQ_LVL or BGOUT)
begin
   case(REQ_LVL)
      2'b00:
         MY_BGOUT <= BGOUT[0];
      2'b01:
         MY_BGOUT <= BGOUT[1];
      2'b10:
         MY_BGOUT <= BGOUT[2];
      2'b11:
         MY_BGOUT <= BGOUT[3];
      default:
         MY_BGOUT <= BGOUT[3];
   endcase
end

always @(REQ_LVL or MON_BGIN)
begin
   case(REQ_LVL)
      2'b00:
         MY_BGIN <= MON_BGIN[0];
      2'b01:
         MY_BGIN <= MON_BGIN[1];
      2'b10:
         MY_BGIN <= MON_BGIN[2];
      2'b11:
         MY_BGIN <= MON_BGIN[3];
      default:
         MY_BGIN <= MON_BGIN[3];
   endcase
end

always @(REQ_LVL or MON_BR)
begin
   case(REQ_LVL)
      2'b00:
         MY_BR <= MON_BR[0];
      2'b01:
         MY_BR <= MON_BR[1];
      2'b10:
         MY_BR <= MON_BR[2];
      2'b11:
         MY_BR <= MON_BR[3];
      default:
         MY_BR <= MON_BR[3];
   endcase
end

always @(posedge SYSCLK)
begin
   DMBR[0] <= MY_BR;
   DMBR[1] <= DMBR[0];
   DMBR[2] <= DMBR[1];
   DMBR[3] <= DMBR[2];
   DMBR[4] <= DMBR[3];
   DMBR[5] <= DMBR[4];
   DMBR[6] <= DMBR[5];
   DMBR[7] <= DMBR[6];
   DMBR[8] <= DMBR[7];
   DMBR[9] <= DMBR[8];
end

assign FAIR_ENA = !(|DMBR);
   
// Bus Request Set Logic and local "BUS_GRANTED" logic   
always @(posedge SYSCLK)
begin
   if(ENA)
      begin
         BUS_GRANTED <= D_BUS_REQ && BBSY;
         if(REQ_TYP[1])          // Fair Requester
            SET_BR <= FAIR_ENA && D_BUS_REQ && !BBSY && !D32_BBSY;
         else                    // Not a Fair Requester
            SET_BR <= D_BUS_REQ && !BBSY && !D32_BBSY;
      end
   else
      begin
         SET_BR <= 0;
         BUS_GRANTED <= 0;
      end
end   

// Bus Request Logic  

always @(posedge SYSCLK or posedge RST)
begin
   if(RST)
      BR <= 4'h0;
   else
      if (SET_BR)
         case(REQ_LVL)
            2'b00:
               BR <= 4'h1;
            2'b01:
               BR <= 4'h2;
            2'b10:
               BR <= 4'h4;
            2'b11:
               BR <= 4'h8;
            default:
               BR <= 4'h8;
         endcase
      else if (D32_BBSY && (|BR))
         BR <= 4'h0;
      else
         BR <= BR; 
end


// Bus Grant Out Logic 

always @(posedge SYSCLK)
begin
   S_BGIN <= MY_BGIN;
   D_BGIN <= S_BGIN;
   S_BUS_REQ <= BUS_REQ && !MY_BGOUT && !S_BGIN;
end
always @(posedge SYSCLK or posedge RST)
begin
   if(RST)
      L_BUS_REQ <= 0;
   else
      if (S_BUS_REQ && S_BGIN)
         L_BUS_REQ <= 1;
      else if (BBSY && !D_BGIN)
         L_BUS_REQ <= 0;
      else
         L_BUS_REQ <= L_BUS_REQ; 
end
  
always @(posedge SYSCLK)
begin
   if(ENA)
      begin
         if(L_BUS_REQ)
            begin
               DRV_BGOUT <= 0;
               LOCAL_GRANT <= D_BGIN;
            end
         else
            begin
               DRV_BGOUT <= D_BGIN;
               LOCAL_GRANT <= 0;
            end
      end
   else
      begin
         DRV_BGOUT <= MY_BGIN;
         LOCAL_GRANT <= 0;
      end
end   

always @(REQ_LVL or DRV_BGOUT or MON_BGIN)
begin
   case(REQ_LVL)
      2'b00:
         begin
            BGOUT[0] <= DRV_BGOUT;
            BGOUT[1] <= MON_BGIN[1];
            BGOUT[2] <= MON_BGIN[2];
            BGOUT[3] <= MON_BGIN[3];
         end
      2'b01:
         begin
            BGOUT[0] <= MON_BGIN[0];
            BGOUT[1] <= DRV_BGOUT;
            BGOUT[2] <= MON_BGIN[2];
            BGOUT[3] <= MON_BGIN[3];
         end
      2'b10:
         begin
            BGOUT[0] <= MON_BGIN[0];
            BGOUT[1] <= MON_BGIN[1];
            BGOUT[2] <= DRV_BGOUT;
            BGOUT[3] <= MON_BGIN[3];
         end
      2'b11:
         begin
            BGOUT[0] <= MON_BGIN[0];
            BGOUT[1] <= MON_BGIN[1];
            BGOUT[2] <= MON_BGIN[2];
            BGOUT[3] <= DRV_BGOUT;
         end
      default:
         begin
            BGOUT[0] <= MON_BGIN[0];
            BGOUT[1] <= MON_BGIN[1];
            BGOUT[2] <= MON_BGIN[2];
            BGOUT[3] <= DRV_BGOUT;
         end
   endcase
end



// Bus Busy Logic
   
always @(posedge SYSCLK)
begin
   if(ENA)
      begin
         if (LOCAL_GRANT)
            BBSY <= 1;
         else if (RELEASE_BBSY)
            BBSY <= 0;
         else
            BBSY <= BBSY;
      end
   else
      BBSY <= 0;
end

// Ensure timing requirements of release of BBSY
assign RDY_4_REL = BBSY_HOLD_90 && BR_REL_D30 && !MY_BGIN && !BUS_REQ;
assign BR_REL_D30 = !(|BR) && !D32_BR;
assign BBSY_HOLD_90 =  BBSY && D92_BBSY;

  
always @(REQ_TYP or RDY_4_REL or MON_BR or RST or MON_SYSRESET)
begin
   if(RST || MON_SYSRESET)
      RELEASE_BBSY <= 1;
   else
      if(REQ_TYP[0])                  // Release When Done (RWD) Requester
         RELEASE_BBSY <= RDY_4_REL;
      else                         // Release On Request (ROR) Requester
         RELEASE_BBSY <= RDY_4_REL && (|MON_BR);
end   

always @(posedge SYSCLK)
begin
   D_BUS_REQ <= BUS_REQ;
   DR0 <= (|BR);
   DR1 <= DR0;
   DR2 <= DR1;
   DR3 <= DR2;
   DR4 <= DR3;
   DR5 <= DR4;
   DR6 <= DR5;
   DR7 <= DR6;
   D32_BR <= DR7;
	DB0 <= BBSY;
   DB1 <= DB0;
   DB2 <= DB1;
   DB3 <= DB2;
   DB4 <= DB3;
   DB5 <= DB4;
   DB6 <= DB5;
   DB7 <= DB6;
   D32_BBSY <= DB7;
   DH0 <= BBSY;
   DH1 <= DH0;
   DH2 <= DH1;
   DH3 <= DH2;
   DH4 <= DH3;
   DH5 <= DH4;
   DH6 <= DH5;
   DH7 <= DH6;
   DH8 <= DH7;
   DH9 <= DH8;
   DH10 <= DH9;
   DH11 <= DH10;
   DH12 <= DH11;
   DH13 <= DH12;
   DH14 <= DH13;
   DH15 <= DH14;
   DH16 <= DH15;
   DH17 <= DH16;
   DH18 <= DH17;
   DH19 <= DH18;
   DH20 <= DH19;
   DH21 <= DH20;
   DH22 <= DH21;
   D92_BBSY <= DH22;
end


endmodule
