// Verilog model  - Wed Feb 25 15:56:32 2004

`timescale 1ns / 1ps

module payload_decode(CLK, RXRESET, DATAIN, FRAME_LEN, FF_RDY, FRAME_REQ, PASS_THRU, MT, READ, ENA, DATAOUT,
      OPCODE, FRAME_READY, MAC_POP, MAC_PUSH, PRIO_FRAME, FF_RD_ERR);

    input CLK;
    input RXRESET;
    input [15:0] DATAIN;
    input [12:0] FRAME_LEN;
    input FF_RDY;
    input FRAME_REQ;
    input PASS_THRU;
    input MT;
   output READ;
    input ENA;
   output [15:0] DATAOUT;
   output [11:0] OPCODE;
   output FRAME_READY;
   output MAC_POP;
   output MAC_PUSH;
   output PRIO_FRAME;
   output FF_RD_ERR;

reg READ;
reg [15:0] DATAOUT;
reg [11:0] OPCODE; 
reg FRAME_READY;
reg FF_RD_ERR;

reg [15:0] PIPE1,PIPE2;
reg [3:0] FF_READ_ST;
reg LOAD_CTRL;
reg [15:0] CTRL;
reg V1,V2,V3;
reg EOD,EOD1,EOD2,EOD3;
reg SET_IMBR;  
reg IMB_READ;  
reg MAC_POP;  
reg MAC_PUSH;  

wire ACK_REQ;
wire IMMEDIATE;

`ifdef HEADER
`else
`include "header.v"
`endif



always @(posedge CLK)
begin: Pipeline
   PIPE1 <= DATAIN;
   PIPE2 <= PIPE1;
   if (ENA)
      DATAOUT <= PIPE2;
   else
      DATAOUT <= 16'hzzzz;
end  

always @(posedge CLK or posedge RXRESET)
begin: FF_READ_FSM
   if (RXRESET)
      FF_READ_ST <= `PL_Idle;
   else
      case (FF_READ_ST)
         `PL_Idle:
            if (FF_RDY && MT)
               FF_READ_ST <= `Rpt_Error;
            else if (FF_RDY && !MT)
               FF_READ_ST <= `Store_Ctrl_Wrd;
            else
               FF_READ_ST <= `PL_Idle; 
         `Store_Ctrl_Wrd:
            FF_READ_ST <= `Wait_4_Frame_Req; 
         `Wait_4_Frame_Req:
            if (FRAME_REQ && MT)
               FF_READ_ST <= `Rpt_Error;
            else if (FRAME_REQ && !MT)
               FF_READ_ST <= `Read_FIFO;
            else
               FF_READ_ST <= `Wait_4_Frame_Req; 
         `Read_FIFO: 
            if (FRAME_LEN == 0)
               FF_READ_ST <= `End_of_Data;
            else if (MT)
               FF_READ_ST <= `Rpt_Error; 
            else
               FF_READ_ST <= `Read_FIFO; 
         `Rpt_Error: 
               FF_READ_ST <= `PL_Idle;
         `End_of_Data: 
               FF_READ_ST <= `Pop1;
         `Pop1:
            if (ACK_REQ || IMB_READ || PASS_THRU)
               FF_READ_ST <= `Push1;
            else
               FF_READ_ST <= `Pop2; 
         `Push1: 
            FF_READ_ST <= `Push2;
         `Push2: 
            FF_READ_ST <= `Push3;
         `Push3: 
            FF_READ_ST <= `PL_Idle;
         `Pop2: 
            FF_READ_ST <= `Pop3;
         `Pop3: 
            FF_READ_ST <= `PL_Idle;
         default: 
            FF_READ_ST <= `Wait_4_Frame_Req;
   endcase
end

always @(FF_READ_ST)
begin: MAC_SORT_OUTS
   case (FF_READ_ST)
      `Pop1:
         begin
            MAC_POP <= 1;
            MAC_PUSH <= 0;
         end
      `Pop2: 
         begin
            MAC_POP <= 1;
            MAC_PUSH <= 0;
         end
      `Pop3: 
         begin
            MAC_POP <= 1;
            MAC_PUSH <= 0;
         end
      `Push1: 
         begin
            MAC_POP <= 1;
            MAC_PUSH <= 1;
         end
      `Push2: 
         begin
            MAC_POP <= 1;
            MAC_PUSH <= 1;
         end
      `Push3: 
         begin
            MAC_POP <= 0;
            MAC_PUSH <= 1;
         end
      default: 
         begin
            MAC_POP <= 0;
            MAC_PUSH <= 0;
         end
   endcase
end

always @(FF_READ_ST or MT or FRAME_LEN)
begin: FSM_OUTS
   case (FF_READ_ST)
      `PL_Idle:
         begin
            READ <= 0;
            LOAD_CTRL <= 0;
            FF_RD_ERR <= 0;
            EOD <= 0;
            FRAME_READY <= 0;
         end
      `Store_Ctrl_Wrd:
         begin
            READ <= 0;
            LOAD_CTRL <= 1;
            FF_RD_ERR <= 0;
            EOD <= 0;
            FRAME_READY <= 0;
         end
      `Wait_4_Frame_Req:
         begin
            READ <= 0;
            LOAD_CTRL <= 0;
            FF_RD_ERR <= 0;
            EOD <= 0;
            FRAME_READY <= 1;
         end
      `Read_FIFO: 
         begin
            READ <= !MT && (FRAME_LEN !=0);
            LOAD_CTRL <= 0;
            FF_RD_ERR <= 0;
            EOD <= 0;
            FRAME_READY <= 0;
         end
      `End_of_Data: 
         begin
            READ <= 0;
            LOAD_CTRL <= 0;
            FF_RD_ERR <= 0;
            EOD <= 1;
            FRAME_READY <= 0;
         end
      `Rpt_Error: 
         begin
            READ <= 0;
            LOAD_CTRL <= 0;
            FF_RD_ERR <= 1;
            EOD <= 0;
            FRAME_READY <= 0;
         end
      default: 
         begin
            READ <= 0;
            LOAD_CTRL <= 0;
            FF_RD_ERR <= 0;
            EOD <= 0;
            FRAME_READY <= 0;
         end
   endcase
end

always @(posedge CLK or posedge RXRESET)
begin
   if (RXRESET)
      CTRL <= 16'h0000;
   else
      if (LOAD_CTRL)
         CTRL <= DATAIN;
      else
         CTRL <= CTRL;
end    

always @(posedge CLK)
begin: Decode
   if(PASS_THRU)
      begin
         OPCODE[7:0] <= `Loopback;
         OPCODE[11:8] <= 4'b0010;
      end
   else
      begin
         OPCODE[7:0] <= CTRL[7:0];
         OPCODE[11:8] <= {1'b0,V3,IMMEDIATE,EOD3};
      end
end
  
always @(posedge CLK)
begin: EOD_Pipe
   EOD1 <= EOD;
   EOD2 <= EOD1;
   EOD3 <= EOD2;
end

always @(posedge CLK)
begin: Imb_Read_Reg
   if (SET_IMBR)
      IMB_READ <= 1;
   else if (EOD3)
      IMB_READ <= 0;
   else
      IMB_READ <= IMB_READ;
end

assign ACK_REQ = CTRL[13];  
assign IMMEDIATE = CTRL[14];  
assign PRIO_FRAME = IMMEDIATE || PASS_THRU;

// SET_IMBR should be an OR of READ bits from each packet type
/*
always @ (PIPE1 or V2)
begin
   SET_IMBR <= (V2 && PIPE1[15]) || (CTRL[7:0] == `Loopback) || (CTRL[7:0] == `Read_Drop_Mem) || (CTRL[7:0] == `Read_WSOP_Mem);       // temporary 
end
*/
always @ (CTRL)
begin
   SET_IMBR <= (CTRL[7:0] == `Loopback) || (CTRL[7:0] == `Read_Drop_Mem) || (CTRL[7:0] == `Read_WSOP_Mem) || (CTRL[7:0] == `Send_N_Words) || (CTRL[7:0] == `Rdbk_Ext_Off) || (CTRL[7:0] == `Rd_Ext_FF) || (CTRL[7:0] == `RT_Ext_FF);       // temporary 
end
always @(posedge CLK)
begin: VPipe
   V1 <= READ;
   V2 <= V1;
   V3 <= V2;
end
  
    
        
endmodule
